@@ -31,6 +31,7 @@ try:
         where,
         coarsen,
         insert,
+        delete,
         append,
         shape,
         union1d,
@@ -84,6 +85,13 @@ try:
         fliplr,
         einsum,
         average,
+        triu,
+        tril,
+        tril_indices,
+        tril_indices_from,
+        triu_indices,
+        triu_indices_from,
+        rot90,
     )
     from .reshape import reshape
     from .ufunc import (
@@ -217,6 +225,7 @@ try:
         reduction,
     )
     from .percentile import percentile
+    from . import lib
     from . import ma
     from . import random, linalg, overlap, fft, backends
     from .overlap import map_overlap
@@ -232,8 +241,7 @@ try:
         indices,
         diag,
         eye,
-        triu,
-        tril,
+        tri,
         fromfunction,
         tile,
         repeat,
