@@ -22,12 +22,12 @@ from .core import (
     broadcast_arrays,
     cached_cumsum,
 )
-from .ufunc import rint
+from .ufunc import rint, greater_equal
 from .wrap import empty, ones, zeros, full
 from .utils import AxisError, meta_from_array, zeros_like_safe
 
 
-def empty_like(a, dtype=None, order="C", chunks=None, name=None):
+def empty_like(a, dtype=None, order="C", chunks=None, name=None, shape=None):
     """
     Return a new array with the same shape and type as a given array.
 
@@ -47,6 +47,8 @@ def empty_like(a, dtype=None, order="C", chunks=None, name=None):
     name : str, optional
         An optional keyname for the array. Defaults to hashing the input
         keyword arguments.
+    shape : int or sequence of ints, optional.
+        Overrides the shape of the result.
 
     Returns
     -------
@@ -70,16 +72,18 @@ def empty_like(a, dtype=None, order="C", chunks=None, name=None):
     """
 
     a = asarray(a, name=False)
+    shape, chunks = _get_like_function_shapes_chunks(a, chunks, shape)
     return empty(
-        a.shape,
+        shape,
         dtype=(dtype or a.dtype),
         order=order,
-        chunks=(chunks if chunks is not None else a.chunks),
+        chunks=chunks,
         name=name,
+        meta=a._meta,
     )
 
 
-def ones_like(a, dtype=None, order="C", chunks=None, name=None):
+def ones_like(a, dtype=None, order="C", chunks=None, name=None, shape=None):
     """
     Return an array of ones with the same shape and type as a given array.
 
@@ -99,6 +103,8 @@ def ones_like(a, dtype=None, order="C", chunks=None, name=None):
     name : str, optional
         An optional keyname for the array. Defaults to hashing the input
         keyword arguments.
+    shape : int or sequence of ints, optional.
+        Overrides the shape of the result.
 
     Returns
     -------
@@ -115,16 +121,18 @@ def ones_like(a, dtype=None, order="C", chunks=None, name=None):
     """
 
     a = asarray(a, name=False)
+    shape, chunks = _get_like_function_shapes_chunks(a, chunks, shape)
     return ones(
-        a.shape,
+        shape,
         dtype=(dtype or a.dtype),
         order=order,
-        chunks=(chunks if chunks is not None else a.chunks),
+        chunks=chunks,
         name=name,
+        meta=a._meta,
     )
 
 
-def zeros_like(a, dtype=None, order="C", chunks=None, name=None):
+def zeros_like(a, dtype=None, order="C", chunks=None, name=None, shape=None):
     """
     Return an array of zeros with the same shape and type as a given array.
 
@@ -144,6 +152,8 @@ def zeros_like(a, dtype=None, order="C", chunks=None, name=None):
     name : str, optional
         An optional keyname for the array. Defaults to hashing the input
         keyword arguments.
+    shape : int or sequence of ints, optional.
+        Overrides the shape of the result.
 
     Returns
     -------
@@ -160,16 +170,18 @@ def zeros_like(a, dtype=None, order="C", chunks=None, name=None):
     """
 
     a = asarray(a, name=False)
+    shape, chunks = _get_like_function_shapes_chunks(a, chunks, shape)
     return zeros(
-        a.shape,
+        shape,
         dtype=(dtype or a.dtype),
         order=order,
-        chunks=(chunks if chunks is not None else a.chunks),
+        chunks=chunks,
         name=name,
+        meta=a._meta,
     )
 
 
-def full_like(a, fill_value, order="C", dtype=None, chunks=None, name=None):
+def full_like(a, fill_value, order="C", dtype=None, chunks=None, name=None, shape=None):
     """
     Return a full array with the same shape and type as a given array.
 
@@ -191,6 +203,8 @@ def full_like(a, fill_value, order="C", dtype=None, chunks=None, name=None):
     name : str, optional
         An optional keyname for the array. Defaults to hashing the input
         keyword arguments.
+    shape : int or sequence of ints, optional.
+        Overrides the shape of the result.
 
     Returns
     -------
@@ -209,16 +223,32 @@ def full_like(a, fill_value, order="C", dtype=None, chunks=None, name=None):
     """
 
     a = asarray(a, name=False)
+    shape, chunks = _get_like_function_shapes_chunks(a, chunks, shape)
     return full(
-        a.shape,
+        shape,
         fill_value,
-        order=order,
         dtype=(dtype or a.dtype),
-        chunks=(chunks if chunks is not None else a.chunks),
+        order=order,
+        chunks=chunks,
         name=name,
+        meta=a._meta,
     )
 
 
+def _get_like_function_shapes_chunks(a, chunks, shape):
+    """
+    Helper function for finding shapes and chunks for *_like()
+    array creation functions.
+    """
+    if shape is None:
+        shape = a.shape
+        if chunks is None:
+            chunks = a.chunks
+    elif chunks is None:
+        chunks = "auto"
+    return shape, chunks
+
+
 def linspace(
     start, stop, num=50, endpoint=True, retstep=False, chunks="auto", dtype=None
 ):
@@ -348,6 +378,9 @@ def arange(*args, **kwargs):
 
     num = int(max(np.ceil((stop - start) / step), 0))
 
+    like = kwargs.pop("like", None)
+    meta = meta_from_array(like) if like is not None else None
+
     dtype = kwargs.pop("dtype", None)
     if dtype is None:
         dtype = np.arange(start, stop, step * num if num else step).dtype
@@ -364,11 +397,18 @@ def arange(*args, **kwargs):
     for i, bs in enumerate(chunks[0]):
         blockstart = start + (elem_count * step)
         blockstop = start + ((elem_count + bs) * step)
-        task = (chunk.arange, blockstart, blockstop, step, bs, dtype)
+        task = (
+            partial(chunk.arange, like=like),
+            blockstart,
+            blockstop,
+            step,
+            bs,
+            dtype,
+        )
         dsk[(name, i)] = task
         elem_count += bs
 
-    return Array(dsk, name, chunks, dtype=dtype)
+    return Array(dsk, name, chunks, dtype=dtype, meta=meta)
 
 
 @derived_from(np)
@@ -646,110 +686,24 @@ def diagonal(a, offset=0, axis1=0, axis2=1):
     return Array(graph, name, shape=shape, chunks=chunks, meta=meta)
 
 
-def triu(m, k=0):
-    """
-    Upper triangle of an array with elements below the `k`-th diagonal zeroed.
+@derived_from(np)
+def tri(N, M=None, k=0, dtype=float, chunks="auto", like=None):
+    _min_int = np.lib.twodim_base._min_int
 
-    Parameters
-    ----------
-    m : array_like, shape (M, N)
-        Input array.
-    k : int, optional
-        Diagonal below which to zero elements.  `k = 0` (the default) is the
-        main diagonal, `k < 0` is below it and `k > 0` is above.
+    if M is None:
+        M = N
 
-    Returns
-    -------
-    triu : ndarray, shape (M, N)
-        Upper triangle of `m`, of same shape and data-type as `m`.
+    chunks = normalize_chunks(chunks, shape=(N, M), dtype=dtype)
 
-    See Also
-    --------
-    tril : lower triangle of an array
-    """
-    if m.ndim != 2:
-        raise ValueError("input must be 2 dimensional")
-    if m.chunks[0][0] != m.chunks[1][0]:
-        msg = (
-            "chunks must be a square. "
-            "Use .rechunk method to change the size of chunks."
-        )
-        raise NotImplementedError(msg)
+    m = greater_equal(
+        arange(N, chunks=chunks[0][0], dtype=_min_int(0, N), like=like).reshape(1, N).T,
+        arange(-k, M - k, chunks=chunks[1][0], dtype=_min_int(-k, M - k), like=like),
+    )
 
-    rdim = len(m.chunks[0])
-    hdim = len(m.chunks[1])
-    chunk = m.chunks[0][0]
+    # Avoid making a copy if the requested type is already bool
+    m = m.astype(dtype, copy=False)
 
-    token = tokenize(m, k)
-    name = "triu-" + token
-
-    dsk = {}
-    for i in range(rdim):
-        for j in range(hdim):
-            if chunk * (j - i + 1) < k:
-                dsk[(name, i, j)] = (
-                    partial(zeros_like_safe, shape=(m.chunks[0][i], m.chunks[1][j])),
-                    m._meta,
-                )
-            elif chunk * (j - i - 1) < k <= chunk * (j - i + 1):
-                dsk[(name, i, j)] = (np.triu, (m.name, i, j), k - (chunk * (j - i)))
-            else:
-                dsk[(name, i, j)] = (m.name, i, j)
-    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[m])
-    return Array(graph, name, shape=m.shape, chunks=m.chunks, meta=m)
-
-
-def tril(m, k=0):
-    """
-    Lower triangle of an array with elements above the `k`-th diagonal zeroed.
-
-    Parameters
-    ----------
-    m : array_like, shape (M, M)
-        Input array.
-    k : int, optional
-        Diagonal above which to zero elements.  `k = 0` (the default) is the
-        main diagonal, `k < 0` is below it and `k > 0` is above.
-
-    Returns
-    -------
-    tril : ndarray, shape (M, M)
-        Lower triangle of `m`, of same shape and data-type as `m`.
-
-    See Also
-    --------
-    triu : upper triangle of an array
-    """
-    if m.ndim != 2:
-        raise ValueError("input must be 2 dimensional")
-    if not len(set(m.chunks[0] + m.chunks[1])) == 1:
-        msg = (
-            "All chunks must be a square matrix to perform lu decomposition. "
-            "Use .rechunk method to change the size of chunks."
-        )
-        raise ValueError(msg)
-
-    rdim = len(m.chunks[0])
-    hdim = len(m.chunks[1])
-    chunk = m.chunks[0][0]
-
-    token = tokenize(m, k)
-    name = "tril-" + token
-
-    dsk = {}
-    for i in range(rdim):
-        for j in range(hdim):
-            if chunk * (j - i + 1) < k:
-                dsk[(name, i, j)] = (m.name, i, j)
-            elif chunk * (j - i - 1) < k <= chunk * (j - i + 1):
-                dsk[(name, i, j)] = (np.tril, (m.name, i, j), k - (chunk * (j - i)))
-            else:
-                dsk[(name, i, j)] = (
-                    partial(zeros_like_safe, shape=(m.chunks[0][i], m.chunks[1][j])),
-                    m._meta,
-                )
-    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[m])
-    return Array(graph, name, shape=m.shape, chunks=m.chunks, meta=m)
+    return m
 
 
 def _np_fromfunction(func, shape, dtype, offset, func_kwargs):
@@ -922,6 +876,8 @@ def linear_ramp_chunk(start, stop, num, dim, step):
     Helper function to find the linear ramp for a chunk.
     """
 
+    from .utils import empty_like_safe
+
     num1 = num + 1
 
     shape = list(start.shape)
@@ -930,7 +886,7 @@ def linear_ramp_chunk(start, stop, num, dim, step):
 
     dtype = np.dtype(start.dtype)
 
-    result = np.empty(shape, dtype=dtype)
+    result = empty_like_safe(start, shape, dtype=dtype)
     for i in np.ndindex(start.shape):
         j = list(i)
         j[dim] = slice(None)
@@ -956,8 +912,13 @@ def pad_edge(array, pad_width, mode, **kwargs):
         pad_arrays = [result, result]
 
         if mode == "constant":
+            from .utils import asarray_safe
+
             constant_values = kwargs["constant_values"][d]
-            constant_values = [asarray(c).astype(result.dtype) for c in constant_values]
+            constant_values = [
+                asarray_safe(c, like=meta_from_array(array), dtype=result.dtype)
+                for c in constant_values
+            ]
 
             pad_arrays = [
                 broadcast_to(v, s, c)
@@ -995,7 +956,7 @@ def pad_edge(array, pad_width, mode, **kwargs):
                 ]
         elif mode == "empty":
             pad_arrays = [
-                empty(s, dtype=array.dtype, chunks=c)
+                empty_like(array, shape=s, dtype=array.dtype, chunks=c)
                 for s, c in zip(pad_shapes, pad_chunks)
             ]
 
@@ -1194,8 +1155,8 @@ def pad(array, pad_width, mode="constant", **kwargs):
     }
     try:
         unsupported_kwargs = set(kwargs) - set(allowed_kwargs[mode])
-    except KeyError:
-        raise ValueError("mode '{}' is not supported".format(mode))
+    except KeyError as e:
+        raise ValueError("mode '{}' is not supported".format(mode)) from e
     if unsupported_kwargs:
         raise ValueError(
             "unsupported keyword arguments for mode '{}': {}".format(
