@@ -240,6 +240,9 @@ def apply_gufunc(func, signature, *args, **kwargs):
         dimensions are to consist only of one chunk.
         Warning: enabling this can increase memory usage significantly.
         Defaults to ``False``.
+    meta: Optional, tuple, keyword only
+        tuple of empty ndarrays describing the shape and dtype of the output of the gufunc.
+        Defaults to ``None``.
     **kwargs : dict
         Extra keyword arguments to pass to `func`
 
@@ -279,6 +282,7 @@ def apply_gufunc(func, signature, *args, **kwargs):
     output_sizes = kwargs.pop("output_sizes", None)
     vectorize = kwargs.pop("vectorize", None)
     allow_rechunk = kwargs.pop("allow_rechunk", False)
+    meta = kwargs.pop("meta", None)
 
     # Input processing:
     ## Signature
@@ -387,7 +391,7 @@ def apply_gufunc(func, signature, *args, **kwargs):
     ### Assert correct partitioning, for case:
     for dim, sizes in dimsizess.items():
         #### Check that the arrays have same length for same dimensions or dimension `1`
-        if set(sizes).union({1}) != {1, max(sizes)}:
+        if set(sizes) | {1} != {1, max(sizes)}:
             raise ValueError(
                 "Dimension `'{}'` with different lengths in arrays".format(dim)
             )
@@ -417,27 +421,32 @@ significantly.".format(
     ### Use existing `blockwise` but only with loopdims to enforce
     ### concatenation for coredims that appear also at the output
     ### Modifying `blockwise` could improve things here.
-    try:
-        tmp = blockwise(  # First try to compute meta
-            func, loop_output_dims, *arginds, concatenate=True, **kwargs
-        )
-    except ValueError:
-        # If computing meta doesn't work, provide it explicitly based on
-        # provided dtypes
-        sample = arginds[0]._meta
-        if isinstance(output_dtypes, tuple):
-            meta = tuple(
-                meta_from_array(sample, dtype=odt)
-                for ocd, odt in zip(output_coredimss, output_dtypes)
-            )
-        else:
-            meta = tuple(
-                meta_from_array(sample, dtype=odt)
-                for ocd, odt in zip((output_coredimss,), (output_dtypes,))
-            )
+    if meta is not None:
         tmp = blockwise(
             func, loop_output_dims, *arginds, concatenate=True, meta=meta, **kwargs
         )
+    else:
+        try:
+            tmp = blockwise(  # First try to compute meta
+                func, loop_output_dims, *arginds, concatenate=True, **kwargs
+            )
+        except ValueError:
+            # If computing meta doesn't work, provide it explicitly based on
+            # provided dtypes
+            sample = arginds[0]._meta
+            if isinstance(output_dtypes, tuple):
+                meta = tuple(
+                    meta_from_array(sample, dtype=odt)
+                    for ocd, odt in zip(output_coredimss, output_dtypes)
+                )
+            else:
+                meta = tuple(
+                    meta_from_array(sample, dtype=odt)
+                    for ocd, odt in zip((output_coredimss,), (output_dtypes,))
+                )
+            tmp = blockwise(
+                func, loop_output_dims, *arginds, concatenate=True, meta=meta, **kwargs
+            )
 
     if isinstance(tmp._meta, tuple):
         metas = tmp._meta
@@ -494,7 +503,7 @@ significantly.".format(
     return (*leaf_arrs,) if nout else leaf_arrs[0]  # Undo *) from above
 
 
-class gufunc(object):
+class gufunc:
     """
     Binds `pyfunc` into ``dask.array.apply_gufunc`` when called.
 
@@ -551,6 +560,9 @@ class gufunc(object):
         dimensions are to consist only of one chunk.
         Warning: enabling this can increase memory usage significantly.
         Defaults to ``False``.
+    meta: Optional, tuple, keyword only
+        tuple of empty ndarrays describing the shape and dtype of the output of the gufunc.
+        Defaults to ``None``.
 
     Returns
     -------
@@ -577,6 +589,17 @@ class gufunc(object):
     >>> c.compute().shape
     (10, 20, 30, 40)
 
+    >>> a = da.ones((1, 5, 10), chunks=(-1, -1, -1))
+    >>> def stats(x):
+    ...     return np.atleast_1d(x.mean()), np.atleast_1d(x.max())
+    >>> meta = (np.array((), dtype=np.float64), np.array((), dtype=np.float64))
+    >>> gustats = da.gufunc(stats, signature="(i,j)->(),()", meta=meta)
+    >>> result = gustats(a)
+    >>> result[0].compute().shape
+    (1,)
+    >>> result[1].compute().shape
+    (1,)
+
     References
     ----------
     .. [1] https://docs.scipy.org/doc/numpy/reference/ufuncs.html
@@ -593,6 +616,7 @@ class gufunc(object):
         self.output_sizes = kwargs.pop("output_sizes", None)
         self.output_dtypes = kwargs.pop("output_dtypes", None)
         self.allow_rechunk = kwargs.pop("allow_rechunk", False)
+        self.meta = kwargs.pop("meta", None)
         if kwargs:
             raise TypeError("Unsupported keyword argument(s) provided")
 
@@ -628,7 +652,8 @@ class gufunc(object):
             output_sizes=self.output_sizes,
             output_dtypes=self.output_dtypes,
             allow_rechunk=self.allow_rechunk or kwargs.pop("allow_rechunk", False),
-            **kwargs
+            meta=self.meta,
+            **kwargs,
         )
 
 
@@ -681,6 +706,9 @@ def as_gufunc(signature=None, **kwargs):
         dimensions are to consist only of one chunk.
         Warning: enabling this can increase memory usage significantly.
         Defaults to ``False``.
+    meta: Optional, tuple, keyword only
+        tuple of empty ndarrays describing the shape and dtype of the output of the gufunc.
+        Defaults to ``None``.
 
     Returns
     -------
@@ -720,8 +748,9 @@ def as_gufunc(signature=None, **kwargs):
         "output_sizes",
         "output_dtypes",
         "allow_rechunk",
+        "meta",
     }
-    if set(_allowedkeys).issubset(kwargs.keys()):
+    if kwargs.keys() - _allowedkeys:
         raise TypeError("Unsupported keyword argument(s) provided")
 
     def _as_gufunc(pyfunc):
