@@ -37,8 +37,8 @@ from dask import delayed
 
 try:
     import scipy.stats
-except ImportError:
-    raise ImportError("`dask.array.stats` requires `scipy` to be installed.")
+except ImportError as e:
+    raise ImportError("`dask.array.stats` requires `scipy` to be installed.") from e
 from scipy.stats import distributions
 from scipy import special
 from scipy.stats.stats import (
@@ -203,9 +203,8 @@ def skew(a, axis=0, bias=True, nan_policy="propagate"):
         raise NotImplementedError("bias=False is not implemented.")
 
     if vals.ndim == 0:
-        return vals
-        # TODO: scalar
-        # return vals.item()
+        # TODO: scalar, min is a workaround
+        return vals.min()
 
     return vals
 
@@ -264,8 +263,11 @@ def kurtosis(a, axis=0, fisher=True, bias=True, nan_policy="propagate"):
     if fisher:
         return vals - 3
     else:
+        if vals.ndim == 0:
+            # TODO: scalar, min is a workaround
+            return vals.min()
+
         return vals
-        # TODO: scalar; vals = vals.item()  # array scalar
 
 
 @derived_from(scipy.stats)
