@@ -173,6 +173,7 @@ def test_apply_gufunc_02():
     a = da.random.normal(size=(20, 30), chunks=(5, 30))
     b = da.random.normal(size=(10, 1, 40), chunks=(10, 1, 40))
     c = apply_gufunc(outer_product, "(i),(j)->(i,j)", a, b, output_dtypes=a.dtype)
+
     assert c.compute().shape == (10, 20, 30, 40)
 
 
@@ -613,3 +614,34 @@ def test_preserve_meta_type():
 
     assert_eq(sum, sum)
     assert_eq(mean, mean)
+
+
+def test_apply_gufunc_with_meta():
+    def stats(x):
+        return np.mean(x, axis=-1), np.std(x, axis=-1, dtype=np.float32)
+
+    a = da.random.normal(size=(10, 20, 30), chunks=(5, 5, 30))
+    meta = (np.ones(0, dtype=np.float64), np.ones(0, dtype=np.float32))
+    result = apply_gufunc(stats, "(i)->(),()", a, meta=meta)
+    expected = stats(a.compute())
+    assert_eq(expected[0], result[0])
+    assert_eq(expected[1], result[1])
+
+
+def test_as_gufunc_with_meta():
+    stack = da.ones((1, 50, 60), chunks=(1, -1, -1))
+    expected = (stack, stack.max())
+
+    meta = (np.array((), dtype=np.float64), np.array((), dtype=np.float64))
+
+    @da.as_gufunc(signature="(i,j) ->(i,j), ()", meta=meta)
+    def array_and_max(arr):
+        return arr, np.atleast_1d(arr.max())
+
+    result = array_and_max(stack)
+    assert_eq(expected[0], result[0])
+
+    # Because `np.max` returns a scalar instead of an `np.ndarray`, we cast
+    # the expected output to a `np.ndarray`, as `meta` defines the output
+    # should be.
+    assert_eq(np.array([expected[1].compute()]), result[1].compute())
