@@ -5,10 +5,17 @@ pytest.importorskip("numpy")
 import numpy as np
 import dask
 import dask.array as da
+from dask.highlevelgraph import HighLevelGraph
 from dask.optimization import fuse
 from dask.utils import SerializableLock
 from dask.array.core import getter, getter_nofancy
-from dask.array.optimization import getitem, optimize, optimize_slices, fuse_slice
+from dask.array.optimization import (
+    getitem,
+    optimize,
+    optimize_blockwise,
+    optimize_slices,
+    fuse_slice,
+)
 from dask.array.utils import assert_eq
 
 
@@ -264,7 +271,8 @@ def test_dont_fuse_numpy_arrays():
 
 
 def test_minimize_data_transfer():
-    x = np.ones(100)
+    zarr = pytest.importorskip("zarr")
+    x = zarr.ones((100,))
     y = da.from_array(x, chunks=25)
     z = y + 1
     dsk = z.__dask_optimize__(z.dask, z.__dask_keys__())
@@ -291,7 +299,7 @@ def test_fuse_slices_with_alias():
     keys = [("dx2", 0)]
     dsk2 = optimize(dsk, keys)
     assert len(dsk2) == 3
-    fused_key = set(dsk2).difference(["x", ("dx2", 0)]).pop()
+    fused_key = (dsk2.keys() - {"x", ("dx2", 0)}).pop()
     assert dsk2[fused_key] == (getter, "x", (slice(0, 4), 0))
 
 
@@ -356,20 +364,52 @@ def test_remove_no_op_slices_if_get_is_not_getter_or_getter_nofancy(get, remove)
         assert optimize_slices({"a": orig}) == {"a": final}
 
 
-@pytest.mark.xfail(reason="blockwise fusion doesnt respect this, which is ok")
+@pytest.mark.xfail(reason="blockwise fusion does not respect this, which is ok")
 def test_turn_off_fusion():
     x = da.ones(10, chunks=(5,))
     y = da.sum(x + 1 + 2 + 3)
 
     a = y.__dask_optimize__(y.dask, y.__dask_keys__())
 
-    with dask.config.set(fuse_ave_width=0):
+    with dask.config.set({"optimization.fuse.ave-width": 0}):
         b = y.__dask_optimize__(y.dask, y.__dask_keys__())
 
     assert dask.get(a, y.__dask_keys__()) == dask.get(b, y.__dask_keys__())
     assert len(a) < len(b)
 
 
+def test_disable_lowlevel_fusion():
+    """Check that by disabling fusion, the HLG survives through optimizations"""
+
+    with dask.config.set({"optimization.fuse.active": False}):
+        y = da.ones(3, chunks=(3,), dtype="int")
+        optimize = y.__dask_optimize__
+        dsk1 = y.__dask_graph__()
+        dsk2 = optimize(dsk1, y.__dask_keys__())
+        assert isinstance(dsk1, HighLevelGraph)
+        assert isinstance(dsk2, HighLevelGraph)
+        assert dsk1 == dsk2
+        y = y.persist()
+        assert isinstance(y.__dask_graph__(), HighLevelGraph)
+        assert_eq(y, [1] * 3)
+
+
+def test_array_creation_blockwise_fusion():
+    """
+    Check that certain array creation routines work with blockwise and can be
+    fused with other blockwise operations.
+    """
+    x = da.ones(3, chunks=(3,))
+    y = da.zeros(3, chunks=(3,))
+    z = da.full(3, fill_value=2, chunks=(3,))
+    a = x + y + z
+    dsk1 = a.__dask_graph__()
+    assert len(dsk1) == 5
+    dsk2 = optimize_blockwise(dsk1)
+    assert len(dsk2) == 1
+    assert_eq(a, np.full(3, 3))
+
+
 def test_gh3937():
     # test for github issue #3937
     x = da.from_array([1, 2, 3.0], (2,))
@@ -389,3 +429,28 @@ def test_double_dependencies():
     X = da.dot(X, X.T)
 
     assert_eq(X.compute(optimize_graph=False), X)
+
+
+def test_fuse_roots():
+    x = da.ones(10, chunks=(2,))
+    y = da.zeros(10, chunks=(2,))
+    z = (x + 1) + (2 * y ** 2)
+    (zz,) = dask.optimize(z)
+    # assert len(zz.dask) == 5
+    assert sum(map(dask.istask, zz.dask.values())) == 5  # there are some aliases
+    assert_eq(zz, z)
+
+
+def test_fuse_roots_annotations():
+    x = da.ones(10, chunks=(2,))
+    y = da.zeros(10, chunks=(2,))
+
+    with dask.annotate(foo="bar"):
+        y = y ** 2
+
+    z = (x + 1) + (2 * y)
+    hlg = dask.blockwise.optimize_blockwise(z.dask)
+    assert len(hlg.layers) == 3
+    assert {"foo": "bar"} in [l.annotations for l in hlg.layers.values()]
+    za = da.Array(hlg, z.name, z.chunks, z.dtype)
+    assert_eq(za, z)
