@@ -17,8 +17,10 @@ from dask.array.overlap import (
     reflect,
     periodic,
     overlap,
+    ensure_minimum_chunksize,
 )
 from dask.array.utils import assert_eq, same_keys
+from ..lib.stride_tricks import sliding_window_view
 
 
 def test_fractional_slice():
@@ -74,7 +76,7 @@ def test_overlap_internal_asymmetric():
     d = da.from_array(x, chunks=(4, 4))
 
     result = overlap_internal(d, {0: (2, 0), 1: (1, 0)})
-    assert result.chunks == ((6, 4), (5, 4))
+    assert result.chunks == ((4, 6), (4, 5))
 
     expected = np.array(
         [
@@ -245,6 +247,13 @@ def test_overlap():
     assert_eq(g, expected)
     assert same_keys(g, overlap(d, depth={0: 2, 1: 1}, boundary={0: 100, 1: "reflect"}))
 
+    u_depth = np.uint16([2, 1])
+    u_depth = {k: v for k, v in enumerate(u_depth)}
+    g = overlap(d, depth=u_depth, boundary={0: 100, 1: "reflect"})
+    assert g.chunks == ((8, 8), (6, 6))
+    assert_eq(g, expected)
+    assert same_keys(g, overlap(d, depth={0: 2, 1: 1}, boundary={0: 100, 1: "reflect"}))
+
     g = overlap(d, depth={0: 2, 1: 1}, boundary={0: 100, 1: "none"})
     expected = np.array(
         [
@@ -269,6 +278,12 @@ def test_overlap():
     assert_eq(g, expected)
     assert g.chunks == ((8, 8), (5, 5))
 
+    u_depth = np.uint16([2, 1])
+    u_depth = {k: v for k, v in enumerate(u_depth)}
+    g = overlap(d, depth=u_depth, boundary={0: 100, 1: "none"})
+    assert_eq(g, expected)
+    assert g.chunks == ((8, 8), (5, 5))
+
 
 def test_asymmetric_overlap_boundary_exception():
     x = da.arange(10, chunks=5)
@@ -288,6 +303,10 @@ def test_map_overlap():
     assert all([(type(s) is int) for s in y.shape])
     assert_eq(y, np.arange(10) + 5 + 2 + 2)
 
+    x = da.ones((10, 10), chunks=(3, 4))
+    z = x.map_overlap(lambda x: x, depth={1: 5})
+    assert z.chunks[0] == x.chunks[0]  # don't rechunk the first dimension
+
     x = np.arange(16).reshape((4, 4))
     d = da.from_array(x, chunks=(2, 2))
     exp1 = d.map_overlap(lambda x: x + x.size, depth=1, dtype=d.dtype)
@@ -322,11 +341,163 @@ def test_map_overlap():
 )
 def test_map_overlap_no_depth(boundary):
     x = da.arange(10, chunks=5)
-
     y = x.map_overlap(lambda i: i, depth=0, boundary=boundary, dtype=x.dtype)
     assert_eq(y, x)
 
 
+def test_map_overlap_multiarray():
+    # Same ndim, same numblocks, same chunks
+    x = da.arange(10, chunks=5)
+    y = da.arange(10, chunks=5)
+    z = da.map_overlap(lambda x, y: x + y, x, y, depth=1)
+    assert_eq(z, 2 * np.arange(10))
+
+    # Same ndim, same numblocks, different chunks
+    x = da.arange(10, chunks=(2, 3, 5))
+    y = da.arange(10, chunks=(5, 3, 2))
+    z = da.map_overlap(lambda x, y: x + y, x, y, depth=1)
+    assert z.chunks == ((2, 3, 3, 2),)
+    assert_eq(z, 2 * np.arange(10))
+
+    # Same ndim, different numblocks, different chunks
+    x = da.arange(10, chunks=(10,))
+    y = da.arange(10, chunks=(4, 4, 2))
+    z = da.map_overlap(lambda x, y: x + y, x, y, depth=1)
+    assert z.chunks == ((4, 4, 2),)
+    assert_eq(z, 2 * np.arange(10))
+
+    # Different ndim, different numblocks, different chunks
+    x = da.arange(10, chunks=(10,))
+    y = da.arange(10).reshape(1, 10).rechunk((1, (4, 4, 2)))
+    z = da.map_overlap(lambda x, y: x + y, x, y, depth=1)
+    assert z.chunks == ((1,), (4, 4, 2))
+    assert z.shape == (1, 10)
+    assert_eq(z, 2 * np.arange(10)[np.newaxis])
+
+    # Note: checks on arange equality in all of the above help ensure that
+    # trimming is applied appropriately to result chunks (i.e. results
+    # are not somehow shifted)
+
+
+def test_map_overlap_multiarray_defaults():
+    # Check that by default, chunk alignment and arrays of varying dimensionality
+    # are supported by with no effect on result shape
+    # (i.e. defaults are pass-through to map_blocks)
+    x = da.ones((10,), chunks=10)
+    y = da.ones((1, 10), chunks=5)
+    z = da.map_overlap(lambda x, y: x + y, x, y)
+    # func should be called twice and get (5,) and (1, 5) arrays of ones each time
+    assert_eq(z.shape, (1, 10))
+    assert_eq(z.sum(), 20)
+
+
+def test_map_overlap_multiarray_different_depths():
+    x = da.ones(5, dtype="int")
+    y = da.ones(5, dtype="int")
+
+    def run(depth):
+        return da.map_overlap(
+            lambda x, y: x.sum() + y.sum(), x, y, depth=depth, chunks=(0,), trim=False
+        ).compute()
+
+    # Check that the number of elements added
+    # to arrays in overlap works as expected
+    # when depths differ for each array
+    assert_eq(run([0, 0]), 10)
+    assert_eq(run([0, 1]), 12)
+    assert_eq(run([1, 1]), 14)
+    assert_eq(run([1, 2]), 16)
+    assert_eq(run([0, 5]), 20)
+    assert_eq(run([5, 5]), 30)
+
+    # Ensure that depth > chunk size results in error
+    with pytest.raises(ValueError):
+        run([0, 6])
+
+
+def test_map_overlap_multiarray_uneven_numblocks_exception():
+    x = da.arange(10, chunks=(10,))
+    y = da.arange(10, chunks=(5, 5))
+    with pytest.raises(ValueError):
+        # Fail with chunk alignment explicitly disabled
+        da.map_overlap(lambda x, y: x + y, x, y, align_arrays=False).compute()
+
+
+def test_map_overlap_multiarray_block_broadcast():
+    def func(x, y):
+        # Return result with expected padding
+        z = x.size + y.size
+        return np.ones((3, 3)) * z
+
+    # Chunks in trailing dimension will be unified to two chunks of size 6
+    # and block broadcast will allow chunks from x to repeat
+    x = da.ones((12,), chunks=12)  # numblocks = (1,) -> (2, 2) after broadcast
+    y = da.ones((16, 12), chunks=(8, 6))  # numblocks = (2, 2)
+    z = da.map_overlap(func, x, y, chunks=(3, 3), depth=1, trim=True)
+    assert_eq(z, z)
+    assert z.shape == (2, 2)
+    # func call will receive (8,) and (10, 8) arrays for each of 4 blocks
+    assert_eq(z.sum(), 4 * (10 * 8 + 8))
+
+
+def test_map_overlap_multiarray_variadic():
+    # Test overlapping row slices from 3D arrays
+    xs = [
+        # Dim 0 will unify to chunks of size 4 for all:
+        da.ones((12, 1, 1), chunks=((12,), 1, 1)),
+        da.ones((12, 8, 1), chunks=((8, 4), 8, 1)),
+        da.ones((12, 8, 4), chunks=((4, 8), 8, 4)),
+    ]
+
+    def func(*args):
+        return np.array([sum([x.size for x in args])])
+
+    x = da.map_overlap(func, *xs, chunks=(1,), depth=1, trim=False, drop_axis=[1, 2])
+
+    # Each func call should get 4 rows from each array padded by 1 in each dimension
+    size_per_slice = sum([np.pad(x[:4], 1, mode="constant").size for x in xs])
+    assert x.shape == (3,)
+    assert all(x.compute() == size_per_slice)
+
+
+def test_map_overlap_assumes_shape_matches_first_array_if_trim_is_false():
+    # https://github.com/dask/dask/issues/6681
+    x1 = da.ones((10,), chunks=(5, 5))
+    x2 = x1.rechunk(10)
+
+    def oversum(x):
+        return x[2:-2]
+
+    z1 = da.map_overlap(oversum, x1, depth=2, trim=False)
+    assert z1.shape == (10,)
+
+    z2 = da.map_overlap(oversum, x2, depth=2, trim=False)
+    assert z2.shape == (10,)
+
+
+def test_map_overlap_deprecated_signature():
+    def func(x):
+        return np.array(x.sum())
+
+    x = da.ones(3)
+
+    # Old positional signature: func, depth, boundary, trim
+    with pytest.warns(FutureWarning):
+        y = da.map_overlap(x, func, 0, "reflect", True)
+        assert y.compute() == 3
+        assert y.shape == (3,)
+
+    with pytest.warns(FutureWarning):
+        y = da.map_overlap(x, func, 1, "reflect", True)
+        assert y.compute() == 5
+        assert y.shape == (3,)
+
+    with pytest.warns(FutureWarning):
+        y = da.map_overlap(x, func, 1, "reflect", False)
+        assert y.compute() == 5
+        assert y.shape == (3,)
+
+
 def test_nearest_overlap():
     a = np.arange(144).reshape(12, 12).astype(float)
 
@@ -422,7 +593,6 @@ def test_depth_equals_boundary_length():
     assert_array_equal(result, expected)
 
 
-@pytest.mark.xfail
 def test_depth_greater_than_boundary_length():
     expected = np.arange(100).reshape(10, 10)
     darr = da.from_array(expected, chunks=(5, 5))
@@ -447,13 +617,31 @@ def test_depth_greater_than_boundary_length():
     assert_array_equal(result, expected)
 
 
-def test_bad_depth_raises():
-    expected = np.arange(144).reshape(12, 12)
-    darr = da.from_array(expected, chunks=(5, 5))
+@pytest.mark.parametrize(
+    "chunks",
+    [
+        ((5, 5, 2), (5, 5, 2)),
+        ((3, 3, 3, 3), (11, 1)),
+    ],
+)
+def test_depth_greater_than_smallest_chunk_combines_chunks(chunks):
+    a = np.arange(144).reshape(12, 12)
+    darr = da.from_array(a, chunks=chunks)
 
     depth = {0: 4, 1: 2}
+    output = overlap(darr, depth=depth, boundary=1)
 
-    pytest.raises(ValueError, overlap, darr, depth=depth, boundary=1)
+    assert all(c >= depth[0] * 2 for c in output.chunks[0])
+    assert all(c >= depth[1] * 2 for c in output.chunks[1])
+
+
+def test_depth_greater_than_dim():
+    a = np.arange(144).reshape(12, 12)
+    darr = da.from_array(a, chunks=(3, 5))
+
+    depth = {0: 13, 1: 4}
+    with pytest.raises(ValueError, match="The overlapping depth"):
+        overlap(darr, depth=depth, boundary=1)
 
 
 def test_none_boundaries():
@@ -550,3 +738,91 @@ def test_trim_boundry(boundary):
     x_overlaped = da.overlap.overlap(x, 2, boundary=boundary)
     x_trimmed = da.overlap.trim_overlap(x_overlaped, 2, boundary=boundary)
     assert np.all(x == x_trimmed)
+
+
+def test_map_overlap_rechunks_array_if_needed():
+    # https://github.com/dask/dask/issues/6597
+    expected = np.arange(11)
+    x = da.from_array(expected, chunks=5)
+    y = x.map_overlap(lambda x: x, depth=2, boundary=0)
+    assert all(c >= 2 for c in y.chunks[0])
+    assert_eq(y, expected)
+
+
+def test_map_overlap_rechunks_array_along_multiple_dims_if_needed():
+    # https://github.com/dask/dask/issues/6688
+    rand = da.random.random((860, 1024, 1024), chunks=(1, 1024, 1024))
+    filtered = rand.map_overlap(
+        lambda arr: arr,
+        depth=(2, 2, 2),
+        boundary="reflect",
+    )
+    assert all(all(c >= 2 for c in chunks) for chunks in filtered.chunks)
+
+
+@pytest.mark.parametrize(
+    "chunks,expected",
+    [
+        [(10,), (10,)],
+        [(10, 10), (10, 10)],
+        [
+            (10, 10, 1),
+            (10, 11),
+        ],
+        [(20, 20, 20, 1), (20, 20, 11, 10)],
+        [(20, 20, 10, 1), (20, 20, 11)],
+        [(2, 20, 2, 20), (14, 10, 20)],
+        [(1, 1, 1, 1, 7), (11,)],
+        [(20, 20, 2, 20, 20, 2), (20, 12, 10, 20, 12, 10)],
+    ],
+)
+def test_ensure_minimum_chunksize(chunks, expected):
+    actual = ensure_minimum_chunksize(10, chunks)
+    assert actual == expected
+
+
+def test_ensure_minimum_chunksize_raises_error():
+    chunks = (5, 2, 1, 1)
+    with pytest.raises(ValueError, match="overlapping depth 10 is larger than"):
+        ensure_minimum_chunksize(10, chunks)
+
+
+@pytest.mark.parametrize(
+    "shape, chunks, window_shape, axis",
+    [
+        ((6, 7, 8), (6, (2, 2, 2, 1), 4), (3, 2), (1, 2)),  # chunks vary along axis
+        ((40, 30, 2), 5, (3,), (0,)),  # window < chunk
+        ((21,), 3, (7,), (0,)),  # window > chunk
+        ((9,), 3, 3, 0),  # window == chunk, axis is integer
+        ((9,), 3, 3, -1),  # axis=-1
+        ((9,), 3, 3, None),  # axis=None
+        ((9, 8), 3, (2, 4), None),  # axis=None
+        ((9,), 3, (3, 3, 3), (0, 0, 0)),  # axis is repeated
+        ((9,), 3, (3, 3), (0, -1)),  # axis is repeated, with -1
+        ((9,), 3, [3, 3], [0, -1]),  # list instead of tuple
+    ],
+)
+def test_sliding_window_view(shape, chunks, window_shape, axis):
+    from ..numpy_compat import sliding_window_view as np_sliding_window_view
+
+    arr = da.from_array(np.arange(np.prod(shape)).reshape(shape), chunks=chunks)
+    actual = sliding_window_view(arr, window_shape, axis)
+    expected = np_sliding_window_view(arr.compute(), window_shape, axis)
+    assert_eq(expected, actual)
+
+
+@pytest.mark.parametrize(
+    "window_shape, axis",
+    [
+        ((10,), 0),  # window > axis shape
+        ((2,), 3),  # axis > ndim
+        (-1, 0),  # window shape is negative
+        (2, (0, 1)),  # len(window shape) < len(axis)
+        (2, None),  # len(window shape) < len(axis)
+        (0, None),  # window_shape = 0
+    ],
+)
+def test_sliding_window_errors(window_shape, axis):
+    arr = da.zeros((4, 3))
+    with pytest.raises(ValueError):
+        sliding_window_view(arr, window_shape, axis)
