@@ -1,5 +1,6 @@
 import difflib
 import functools
+import itertools
 import math
 import numbers
 import os
@@ -10,7 +11,7 @@ from tlz import frequencies, concat
 
 from .core import Array
 from ..highlevelgraph import HighLevelGraph
-from ..utils import has_keyword, ignoring, is_arraylike
+from ..utils import has_keyword, ignoring, is_arraylike, is_cupy_type
 
 try:
     AxisError = np.AxisError
@@ -21,13 +22,8 @@ except AttributeError:
         AxisError = type(e)
 
 
-def _is_cupy_type(x):
-    # TODO: avoid explicit reference to CuPy
-    return "cupy" in str(type(x))
-
-
 def normalize_to_array(x):
-    if _is_cupy_type(x):
+    if is_cupy_type(x):
         return x.get()
     else:
         return x
@@ -225,7 +221,19 @@ def assert_eq_shape(a, b, check_nan=True):
             assert aa == bb
 
 
-def _get_dt_meta_computed(x, check_shape=True, check_graph=True):
+def _check_chunks(x):
+    x = x.persist(scheduler="sync")
+    for idx in itertools.product(*(range(len(c)) for c in x.chunks)):
+        chunk = x.dask[(x.name,) + idx]
+        if not hasattr(chunk, "dtype"):
+            chunk = np.array(chunk, dtype="O")
+        expected_shape = tuple(c[i] for c, i in zip(x.chunks, idx))
+        assert_eq_shape(expected_shape, chunk.shape, check_nan=False)
+        assert chunk.dtype == x.dtype
+    return x
+
+
+def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=True):
     x_original = x
     x_meta = None
     x_computed = None
@@ -236,6 +244,9 @@ def _get_dt_meta_computed(x, check_shape=True, check_graph=True):
         if check_graph:
             _check_dsk(x.dask)
         x_meta = getattr(x, "_meta", None)
+        if check_chunks:
+            # Replace x with persisted version to avoid computing it twice.
+            x = _check_chunks(x)
         x = x.compute(scheduler="sync")
         x_computed = x
         if hasattr(x, "todense"):
@@ -254,15 +265,23 @@ def _get_dt_meta_computed(x, check_shape=True, check_graph=True):
     return x, adt, x_meta, x_computed
 
 
-def assert_eq(a, b, check_shape=True, check_graph=True, check_meta=True, **kwargs):
+def assert_eq(
+    a,
+    b,
+    check_shape=True,
+    check_graph=True,
+    check_meta=True,
+    check_chunks=True,
+    **kwargs,
+):
     a_original = a
     b_original = b
 
     a, adt, a_meta, a_computed = _get_dt_meta_computed(
-        a, check_shape=check_shape, check_graph=check_graph
+        a, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
     )
     b, bdt, b_meta, b_computed = _get_dt_meta_computed(
-        b, check_shape=check_shape, check_graph=check_graph
+        b, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
     )
 
     if str(adt) != str(bdt):
@@ -344,6 +363,16 @@ def safe_wraps(wrapped, assigned=functools.WRAPPER_ASSIGNMENTS):
         return lambda x: x
 
 
+def _dtype_of(a):
+    """Determine dtype of an array-like."""
+    try:
+        # Check for the attribute before using asanyarray, because some types
+        # (notably sparse arrays) don't work with it.
+        return a.dtype
+    except AttributeError:
+        return np.asanyarray(a).dtype
+
+
 def empty_like_safe(a, shape, **kwargs):
     """
     Return np.empty_like(a, shape=shape, **kwargs) if the shape argument
@@ -353,6 +382,7 @@ def empty_like_safe(a, shape, **kwargs):
     try:
         return np.empty_like(a, shape=shape, **kwargs)
     except TypeError:
+        kwargs.setdefault("dtype", _dtype_of(a))
         return np.empty(shape, **kwargs)
 
 
@@ -366,6 +396,7 @@ def full_like_safe(a, fill_value, shape, **kwargs):
     try:
         return np.full_like(a, fill_value, shape=shape, **kwargs)
     except TypeError:
+        kwargs.setdefault("dtype", _dtype_of(a))
         return np.full(shape, fill_value, **kwargs)
 
 
@@ -378,6 +409,7 @@ def ones_like_safe(a, shape, **kwargs):
     try:
         return np.ones_like(a, shape=shape, **kwargs)
     except TypeError:
+        kwargs.setdefault("dtype", _dtype_of(a))
         return np.ones(shape, **kwargs)
 
 
@@ -390,9 +422,25 @@ def zeros_like_safe(a, shape, **kwargs):
     try:
         return np.zeros_like(a, shape=shape, **kwargs)
     except TypeError:
+        kwargs.setdefault("dtype", _dtype_of(a))
         return np.zeros(shape, **kwargs)
 
 
+def arange_safe(*args, like, **kwargs):
+    """
+    Use the `like=` from `np.arange` to create a new array dispatching
+    to the downstream library. If that fails, falls back to the
+    default NumPy behavior, resulting in a `numpy.ndarray`.
+    """
+    if like is None:
+        return np.arange(*args, **kwargs)
+    else:
+        try:
+            return np.arange(*args, like=meta_from_array(like), **kwargs)
+        except TypeError:
+            return np.arange(*args, **kwargs)
+
+
 def _array_like_safe(np_func, da_func, a, like, **kwargs):
     if like is a and hasattr(a, "__array_function__"):
         return a
@@ -400,7 +448,7 @@ def _array_like_safe(np_func, da_func, a, like, **kwargs):
     if isinstance(like, Array):
         return da_func(a, **kwargs)
     elif isinstance(a, Array):
-        if _is_cupy_type(a._meta):
+        if is_cupy_type(a._meta):
             a = a.compute(scheduler="sync")
 
     try:
@@ -425,6 +473,34 @@ def array_safe(a, like, **kwargs):
     return _array_like_safe(np.array, array, a, like, **kwargs)
 
 
+def asarray_safe(a, like, **kwargs):
+    """
+    If a is dask.array, return dask.array.asarray(a, **kwargs),
+    otherwise return np.asarray(a, like=like, **kwargs), dispatching
+    the call to the library that implements the like array. Note that
+    when a is a dask.Array but like isn't, this function will call
+    a.compute(scheduler="sync") before np.asarray, as downstream
+    libraries are unlikely to know how to convert a dask.Array.
+    """
+    from .core import asarray
+
+    return _array_like_safe(np.asarray, asarray, a, like, **kwargs)
+
+
+def asanyarray_safe(a, like, **kwargs):
+    """
+    If a is dask.array, return dask.array.asanyarray(a, **kwargs),
+    otherwise return np.asanyarray(a, like=like, **kwargs), dispatching
+    the call to the library that implements the like array. Note that
+    when a is a dask.Array but like isn't, this function will call
+    a.compute(scheduler="sync") before np.asanyarray, as downstream
+    libraries are unlikely to know how to convert a dask.Array.
+    """
+    from .core import asanyarray
+
+    return _array_like_safe(np.asanyarray, asanyarray, a, like, **kwargs)
+
+
 def validate_axis(axis, ndim):
     """ Validate an input to axis= keywords """
     if isinstance(axis, (tuple, list)):
