@@ -1,6 +1,10 @@
 import io
 import uuid
 
+from fsspec.core import open_files, get_fs_token_paths, OpenFile
+from fsspec.utils import tokenize as fs_tokenize, read_block
+
+
 from ..highlevelgraph import HighLevelGraph
 
 MAGIC = b"Obj\x01"
@@ -58,8 +62,6 @@ def read_header(fo):
 
 def open_head(fs, path, compression):
     """Open a file just to read its head and size"""
-    from dask.bytes.core import OpenFile
-
     with OpenFile(fs, path, compression=compression) as f:
         head = read_header(f)
     size = fs.info(path)["size"]
@@ -89,7 +91,6 @@ def read_avro(urlpath, blocksize=100000000, storage_options=None, compression=No
     """
     from dask.utils import import_required
     from dask import delayed, compute
-    from dask.bytes.core import open_files, get_fs_token_paths, OpenFile, tokenize
     from dask.bag import from_delayed
 
     import_required(
@@ -118,7 +119,7 @@ def read_avro(urlpath, blocksize=100000000, storage_options=None, compression=No
         for path, offset, length, head in zip(paths, offsets, lengths, heads):
             delimiter = head["sync"]
             f = OpenFile(fs, path, compression=compression)
-            token = tokenize(
+            token = fs_tokenize(
                 fs_token, delimiter, path, fs.ukey(path), compression, offset
             )
             keys = ["read-avro-%s-%s" % (o, token) for o in offset]
@@ -139,7 +140,11 @@ def read_avro(urlpath, blocksize=100000000, storage_options=None, compression=No
 def read_chunk(fobj, off, l, head):
     """Get rows from raw bytes block"""
     import fastavro
-    from dask.bytes.core import read_block
+
+    if hasattr(fastavro, "iter_avro"):
+        reader = fastavro.iter_avro
+    else:
+        reader = fastavro.reader
 
     with fobj as f:
         chunk = read_block(f, off, l, head["sync"])
@@ -147,15 +152,20 @@ def read_chunk(fobj, off, l, head):
     if not chunk.startswith(MAGIC):
         chunk = head_bytes + chunk
     i = io.BytesIO(chunk)
-    return list(fastavro.iter_avro(i))
+    return list(reader(i))
 
 
 def read_file(fo):
     """Get rows from file-like"""
     import fastavro
 
+    if hasattr(fastavro, "iter_avro"):
+        reader = fastavro.iter_avro
+    else:
+        reader = fastavro.reader
+
     with fo as f:
-        return list(fastavro.iter_avro(f))
+        return list(reader(f))
 
 
 def to_avro(
@@ -198,7 +208,7 @@ def to_avro(
     b: dask.bag.Bag
     filename: list of str or str
         Filenames to write to. If a list, number must match the number of
-        partitions. If a string, must includ a glob character "*", which will
+        partitions. If a string, must include a glob character "*", which will
         be expanded using name_function
     schema: dict
         Avro schema dictionary, see above
@@ -234,7 +244,6 @@ def to_avro(
     """
     # TODO infer schema from first partition of data
     from dask.utils import import_required
-    from dask.bytes.core import open_files
 
     import_required(
         "fastavro", "fastavro is a required dependency for using bag.to_avro()."
