@@ -1,9 +1,9 @@
 import pytest
-from toolz import partial
+from functools import partial
+from tlz import concat
 
 import dask
 from dask import compute
-from dask.compatibility import PY_VERSION
 from dask.utils import filetexts
 from dask.bytes import utils
 from dask.bag.text import read_text
@@ -33,13 +33,16 @@ expected = "".join([files[v] for v in sorted(files)])
 fmt_bs = [(fmt, None) for fmt in compr] + [(None, "10 B")]
 
 encodings = ["ascii", "utf-8"]  # + ['utf-16', 'utf-16-le', 'utf-16-be']
-fmt_bs_enc = [(fmt, bs, encoding) for fmt, bs in fmt_bs for encoding in encodings]
+fmt_bs_enc_path = [
+    (fmt, bs, encoding, include_path)
+    for fmt, bs in fmt_bs
+    for encoding in encodings
+    for include_path in (True, False)
+]
 
 
-@pytest.mark.parametrize("fmt,bs,encoding", fmt_bs_enc)
-def test_read_text(fmt, bs, encoding):
-    if fmt == "zip" and PY_VERSION < "3.6":
-        pytest.skip("zipfile is read-only on py35")
+@pytest.mark.parametrize("fmt,bs,encoding,include_path", fmt_bs_enc_path)
+def test_read_text(fmt, bs, encoding, include_path):
     if fmt not in utils.compress:
         pytest.skip("compress function not provided for %s" % fmt)
     compress = utils.compress[fmt]
@@ -51,9 +54,24 @@ def test_read_text(fmt, bs, encoding):
         (L,) = compute(b)
         assert "".join(L) == expected
 
-        b = read_text(sorted(files), compression=fmt, blocksize=bs, encoding=encoding)
+        o = read_text(
+            sorted(files),
+            compression=fmt,
+            blocksize=bs,
+            encoding=encoding,
+            include_path=include_path,
+        )
+        b = o.pluck(0) if include_path else o
         (L,) = compute(b)
         assert "".join(L) == expected
+        if include_path:
+            (paths,) = compute(o.pluck(1))
+            expected_paths = list(
+                concat([[k] * v.count("\n") for k, v in files.items()])
+            )
+            assert len(paths) == len(expected_paths)
+            for path, expected_path in zip(paths, expected_paths):
+                assert path.endswith(expected_path)
 
         blocks = read_text(
             ".test.accounts.*.json",
@@ -66,6 +84,18 @@ def test_read_text(fmt, bs, encoding):
         assert "".join(line for block in L for line in block) == expected
 
 
+def test_read_text_unicode_no_collection():
+    data = b"abcd\xc3\xa9"
+    fn = "./data.txt"
+    with open(fn, "wb") as f:
+        f.write(b"\n".join([data, data]))
+
+    f = read_text(fn, collection=False)
+
+    result = f[0].compute()
+    assert len(result) == 2
+
+
 def test_files_per_partition():
     files3 = {"{:02}.txt".format(n): "line from {:02}" for n in range(20)}
     with filetexts(files3):
@@ -80,6 +110,18 @@ def test_files_per_partition():
 
             assert b.count().compute() == 20, "All 20 lines should be read"
 
+            with pytest.warns(UserWarning):
+                b = read_text("*.txt", files_per_partition=10, include_path=True)
+                p = b.take(100, npartitions=1)
+
+            p_paths = tuple(zip(*p))[1]
+            p_unique_paths = set(p_paths)
+            assert len(p_unique_paths) == 10
+
+            b_paths = tuple(zip(*b.compute()))[1]
+            b_unique_paths = set(b_paths)
+            assert len(b_unique_paths) == 20
+
 
 def test_errors():
     with filetexts({".test.foo": b"Jos\xe9\nAlice"}, mode="b"):
@@ -89,3 +131,21 @@ def test_errors():
         result = read_text(".test.foo", encoding="ascii", errors="ignore")
         result = result.compute(scheduler="sync")
         assert result == ["Jos\n", "Alice"]
+
+
+def test_complex_delimiter():
+    longstr = "abc\ndef\n123\n$$$$\ndog\ncat\nfish\n\n\r\n$$$$hello"
+    with filetexts({".test.delim.txt": longstr}):
+        assert read_text(".test.delim.txt", linedelimiter="$$$$").count().compute() == 3
+        assert (
+            read_text(".test.delim.txt", linedelimiter="$$$$", blocksize=2)
+            .count()
+            .compute()
+            == 3
+        )
+        vals = read_text(".test.delim.txt", linedelimiter="$$$$").compute()
+        assert vals[-1] == "hello"
+        assert vals[0].endswith("$$$$")
+        vals = read_text(".test.delim.txt", linedelimiter="$$$$", blocksize=2).compute()
+        assert vals[-1] == "hello"
+        assert vals[0].endswith("$$$$")
