@@ -1,14 +1,13 @@
-from __future__ import print_function, division, absolute_import
-
 import os
 import posixpath
 
 import pytest
-from toolz import concat
+from tlz import concat
+from fsspec.core import open_files, get_fs_token_paths
 
 import dask
-from dask.bytes.core import read_bytes, open_files, get_fs_token_paths
-from dask.compatibility import unicode
+import dask.bag as db
+from dask.bytes.core import read_bytes
 
 
 try:
@@ -147,7 +146,6 @@ def test_read_csv(hdfs):
 
 
 def test_read_text(hdfs):
-    db = pytest.importorskip("dask.bag")
     import multiprocessing as mp
 
     pool = mp.get_context("spawn").Pool(2)
@@ -176,8 +174,6 @@ def test_read_text(hdfs):
 
 
 def test_read_text_unicode(hdfs):
-    db = pytest.importorskip("dask.bag")
-
     data = b"abcd\xc3\xa9"
     fn = "%s/data.txt" % basedir
     with hdfs.open(fn, "wb") as f:
@@ -187,7 +183,7 @@ def test_read_text_unicode(hdfs):
 
     result = f[0].compute()
     assert len(result) == 2
-    assert list(map(unicode.strip, result)) == [data.decode("utf-8")] * 2
+    assert list(map(str.strip, result)) == [data.decode("utf-8")] * 2
     assert len(result[0].strip()) == 5
 
 
