@@ -1,19 +1,21 @@
-from __future__ import print_function, division, absolute_import
-
 import gzip
 import os
-from time import sleep
+import pathlib
 import sys
+from time import sleep
+from functools import partial
+from distutils.version import LooseVersion
 
 import pytest
-from toolz import concat, valmap, partial
+import cloudpickle
+from tlz import concat, valmap
+from fsspec.core import open_files
 
 from dask import compute
-from dask.compatibility import FileNotFoundError, unicode
 from dask.utils import filetexts
 from fsspec.implementations.local import LocalFileSystem
 from fsspec.compression import compr
-from dask.bytes.core import read_bytes, open_files, get_fs_token_paths
+from dask.bytes.core import read_bytes
 from dask.bytes.utils import compress
 
 compute = partial(compute, scheduler="sync")
@@ -41,62 +43,15 @@ csv_files = {
 }
 
 
-try:
-    # used only in test_with_urls - may be more generally useful
-    import pathlib
-
-    def to_uri(path):
-        return pathlib.Path(os.path.abspath(path)).as_uri()
+def to_uri(path):
+    return pathlib.Path(os.path.abspath(path)).as_uri()
 
 
-except (ImportError, NameError):
-    import urlparse, urllib
+def test_unordered_urlpath_errors():
 
-    def to_uri(path):
-        return urlparse.urljoin("file:", urllib.pathname2url(os.path.abspath(path)))
-
-
-def test_urlpath_inference_strips_protocol(tmpdir):
-    tmpdir = str(tmpdir)
-    paths = [os.path.join(tmpdir, "test.%02d.csv" % i) for i in range(20)]
-
-    for path in paths:
-        with open(path, "wb") as f:
-            f.write(b"1,2,3\n" * 10)
-
-    # globstring
-    protocol = "file:///" if sys.platform == "win32" else "file://"
-    urlpath = protocol + os.path.join(tmpdir, "test.*.csv")
-    _, _, paths2 = get_fs_token_paths(urlpath)
-    assert "file:" not in paths2[0]
-    assert paths2[0].endswith("/test.00.csv")
-
-    # list of paths
-    _, _, paths3 = get_fs_token_paths([protocol + p for p in paths])
-    assert paths2 == paths3
-
-
-def test_urlpath_inference_errors():
-    # Empty list
-    with pytest.raises(ValueError, match="empty"):
-        get_fs_token_paths([])
-
-    # Protocols differ
-    with pytest.raises(ValueError, match="the same protocol"):
-        get_fs_token_paths(["s3://test/path.csv", "/other/path.csv"])
-
-    # Options differ
-    with pytest.raises(ValueError, match="the same file-system options"):
-        get_fs_token_paths(
-            [
-                "ftp://myuser@node.com/test/path.csv",
-                "ftp://otheruser@node.com/other/path.csv",
-            ]
-        )
-
-    # Unknown type
+    # Unordered urlpath argument
     with pytest.raises(TypeError):
-        get_fs_token_paths(
+        read_bytes(
             {
                 "sets/are.csv",
                 "unordered/so/they.csv",
@@ -106,42 +61,6 @@ def test_urlpath_inference_errors():
         )
 
 
-def test_urlpath_expand_read():
-    """Make sure * is expanded in file paths when reading."""
-    # when reading, globs should be expanded to read files by mask
-    with filetexts(csv_files, mode="b"):
-        _, _, paths = get_fs_token_paths(".*.csv")
-        assert len(paths) == 2
-        _, _, paths = get_fs_token_paths([".*.csv"])
-        assert len(paths) == 2
-
-
-@pytest.mark.skipif(
-    sys.version_info < (3, 5), reason="Recursive glob is new in Python 3.5"
-)
-def test_recursive_glob_expand():
-    """Make sure * is expanded in file paths when reading."""
-    with filetexts(
-        {"sub1/afile.csv": b"", "sub1/sub2/another.csv": b"", "sub1/twofile.csv": b""},
-        mode="b",
-    ):
-        _, _, paths = get_fs_token_paths(os.path.abspath("**/*.csv"))
-        assert len(paths) == 3
-
-
-def test_urlpath_expand_write():
-    """Make sure * is expanded in file paths when writing."""
-    _, _, paths = get_fs_token_paths("prefix-*.csv", mode="wb", num=2)
-    assert [p.endswith(pa) for p, pa in zip(paths, ["prefix-0.csv", "prefix-1.csv"])]
-    _, _, paths = get_fs_token_paths(["prefix-*.csv"], mode="wb", num=2)
-    assert [p.endswith(pa) for p, pa in zip(paths, ["prefix-0.csv", "prefix-1.csv"])]
-    # we can read with multiple masks, but not write
-    with pytest.raises(ValueError):
-        _, _, paths = get_fs_token_paths(
-            ["prefix1-*.csv", "prefix2-*.csv"], mode="wb", num=2
-        )
-
-
 def test_read_bytes():
     with filetexts(files, mode="b"):
         sample, values = read_bytes(".test.accounts.*")
@@ -205,7 +124,7 @@ def test_read_bytes_blocksize_float_errs():
 def test_read_bytes_include_path():
     with filetexts(files, mode="b"):
         _, _, paths = read_bytes(".test.accounts.*", include_path=True)
-        assert {os.path.split(path)[1] for path in paths} == set(files.keys())
+        assert {os.path.split(path)[1] for path in paths} == files.keys()
 
 
 def test_with_urls():
@@ -218,7 +137,6 @@ def test_with_urls():
 
 @pytest.mark.skipif(sys.platform == "win32", reason="pathlib and moto clash on windows")
 def test_with_paths():
-    pathlib = pytest.importorskip("pathlib")
     with filetexts(files, mode="b"):
         url = pathlib.Path("./.test.accounts.*")
         sample, values = read_bytes(url, blocksize=None)
@@ -274,8 +192,6 @@ fmt_bs = [(fmt, None) for fmt in compr] + [(fmt, 10) for fmt in compr]
 
 @pytest.mark.parametrize("fmt,blocksize", fmt_bs)
 def test_compression(fmt, blocksize):
-    if fmt == "zip" and sys.version_info.minor == 5:
-        pytest.skip("zipfile is read-only on py35")
     if fmt not in compress:
         pytest.skip("compression function not provided")
     files2 = valmap(compress[fmt], files)
@@ -327,8 +243,6 @@ def test_open_files_text_mode(encoding):
 @pytest.mark.parametrize("mode", ["rt", "rb"])
 @pytest.mark.parametrize("fmt", list(compr))
 def test_open_files_compression(mode, fmt):
-    if fmt == "zip" and sys.version_info.minor == 5:
-        pytest.skip("zipfile is read-only on py35")
     if fmt not in compress:
         pytest.skip("compression function not provided")
     files2 = valmap(compress[fmt], files)
@@ -397,7 +311,6 @@ def test_open_files_write(tmpdir, compression_opener):
 
 def test_pickability_of_lazy_files(tmpdir):
     tmpdir = str(tmpdir)
-    cloudpickle = pytest.importorskip("cloudpickle")
 
     with filetexts(files, mode="b"):
         myfiles = open_files(".test.accounts.*")
@@ -418,7 +331,7 @@ def test_py2_local_bytes(tmpdir):
     files = open_files(fn, compression="gzip", mode="rt")
 
     with files[0] as f:
-        assert all(isinstance(line, unicode) for line in f)
+        assert all(isinstance(line, str) for line in f)
 
 
 def test_abs_paths(tmpdir):
@@ -443,6 +356,8 @@ def test_get_pyarrow_filesystem():
     from fsspec.implementations.local import LocalFileSystem
 
     pa = pytest.importorskip("pyarrow")
+    if pa.__version__ >= LooseVersion("2.0.0"):
+        pytest.skip("fsspec no loger inherits from pyarrow>=2.0.")
 
     fs = LocalFileSystem()
     assert isinstance(fs, pa.filesystem.FileSystem)
