@@ -7,8 +7,6 @@ from warnings import warn
 import pandas as pd
 from tlz import merge
 
-# this import checks for the importability of fsspec
-from ...bytes import read_bytes  # noqa
 from fsspec.utils import build_name_function, stringify_path
 
 from .io import _link
@@ -47,7 +45,7 @@ def to_hdf(
     dask_kwargs={},
     **kwargs
 ):
-    """ Store Dask Dataframe to Hierarchical Data Format (HDF) files
+    """Store Dask Dataframe to Hierarchical Data Format (HDF) files
 
     This is a parallel version of the Pandas function of the same name.  Please
     see the Pandas docstring for more detailed information about shared keyword
@@ -296,7 +294,21 @@ def _read_single_hdf(
         key.
         """
         with pd.HDFStore(path, mode=mode) as hdf:
-            keys = [k for k in hdf.keys() if fnmatch(k, key)]
+            import glob
+
+            if not glob.has_magic(key):
+                keys = [key]
+            else:
+                keys = [k for k in hdf.keys() if fnmatch(k, key)]
+                # https://github.com/dask/dask/issues/5934
+                # TODO: remove this part if/when pandas copes with all keys
+                keys.extend(
+                    n._v_pathname
+                    for n in hdf._handle.walk_nodes("/", classname="Table")
+                    if fnmatch(n._v_pathname, key)
+                    and n._v_name != "table"
+                    and n._v_pathname not in keys
+                )
             stops = []
             divisions = []
             for k in keys:
@@ -405,7 +417,7 @@ def read_hdf(
     chunksize=1000000,
     sorted_index=False,
     lock=True,
-    mode="a",
+    mode="r",
 ):
     """
     Read HDF files into a Dask DataFrame
@@ -433,7 +445,7 @@ def read_hdf(
         index (default is False).
     lock : boolean, optional
         Option to use a lock to prevent concurrency issues (default is True).
-    mode : {'a', 'r', 'r+'}, default 'a'. Mode to use when opening file(s).
+    mode : {'a', 'r', 'r+'}, default 'r'. Mode to use when opening file(s).
         'r'
             Read-only; no data can be modified.
         'a'
