@@ -20,9 +20,9 @@ from .utils import (
     _parse_pandas_metadata,
     _normalize_index_columns,
     Engine,
-    _analyze_paths,
     _flatten_filters,
     _row_groups_to_parts,
+    _sort_and_analyze_paths,
 )
 
 
@@ -106,7 +106,11 @@ def _write_partitioned(
             ]
         )
         subtable = pa.Table.from_pandas(
-            subgroup, preserve_index=preserve_index, schema=subschema, safe=False
+            subgroup,
+            nthreads=1,
+            preserve_index=preserve_index,
+            schema=subschema,
+            safe=False,
         )
         prefix = fs.sep.join([root_path, subdir])
         fs.mkdirs(prefix, exist_ok=True)
@@ -246,7 +250,7 @@ def _collect_pyarrow_dataset_frags(
             ]
 
         # Append fragments to our "metadata" list
-        if ds_filters:
+        if ds_filters is not None:
             # If we have filters, we need to split the row groups to apply them.
             # If any row-groups are filtered out, we convert the remaining row-groups
             # to a NEW (filtered) fragment, and append the filtered fragment to our
@@ -661,9 +665,7 @@ class ArrowDatasetEngine(Engine):
             elif index_in_columns_and_parts:
                 # The correct index has already been set
                 index = False
-                columns_and_parts = list(
-                    set(columns_and_parts).difference(set(df.index.names))
-                )
+                columns_and_parts = list(set(columns_and_parts) - set(df.index.names))
         df = df[list(columns_and_parts)]
 
         if index:
@@ -813,7 +815,9 @@ class ArrowDatasetEngine(Engine):
     def _pandas_to_arrow_table(
         cls, df: pd.DataFrame, preserve_index=False, schema=None
     ) -> pa.Table:
-        table = pa.Table.from_pandas(df, preserve_index=preserve_index, schema=schema)
+        table = pa.Table.from_pandas(
+            df, nthreads=1, preserve_index=preserve_index, schema=schema
+        )
         return table
 
     @classmethod
@@ -947,7 +951,7 @@ class ArrowDatasetEngine(Engine):
 
             # Use _analyze_paths to avoid relative-path
             # problems (see GH#5608)
-            base, fns = _analyze_paths(paths, fs)
+            paths, base, fns = _sort_and_analyze_paths(paths, fs)
             paths = fs.sep.join([base, fns[0]])
 
             meta_path = fs.sep.join([paths, "_metadata"])
@@ -964,7 +968,7 @@ class ArrowDatasetEngine(Engine):
                 if gather_statistics is None:
                     gather_statistics = True
         elif len(paths) > 1:
-            base, fns = _analyze_paths(paths, fs)
+            paths, base, fns = _sort_and_analyze_paths(paths, fs)
             meta_path = fs.sep.join([base, "_metadata"])
             if "_metadata" in fns:
                 # Pyarrow cannot handle "_metadata" when `paths` is a list
@@ -1308,7 +1312,11 @@ class ArrowDatasetEngine(Engine):
                 if row_group_info is None:
                     frag.ensure_complete_metadata()
                     row_group_info = frag.row_groups
-                frag_map[(fpath, row_group_info[0].id)] = frag
+                if len(row_group_info):
+                    frag_map[(fpath, row_group_info[0].id)] = frag
+                else:
+                    # All row-groups were filtered
+                    continue
             else:
                 file_row_groups[fpath] = [None]
                 frag_map[(fpath, None)] = frag
@@ -1603,7 +1611,7 @@ def _get_dataset_object(paths, fs, filters, dataset_kwargs):
         kwargs["validate_schema"] = False
     if len(paths) > 1:
         # This is a list of files
-        base, fns = _analyze_paths(paths, fs)
+        paths, base, fns = _sort_and_analyze_paths(paths, fs)
         proxy_metadata = None
         if "_metadata" in fns:
             # We have a _metadata file. PyArrow cannot handle
@@ -1628,7 +1636,7 @@ def _get_dataset_object(paths, fs, filters, dataset_kwargs):
         #       existence of _metadata.  Listing may be much more
         #       expensive in storage systems like S3.
         allpaths = fs.glob(paths[0] + fs.sep + "*")
-        base, fns = _analyze_paths(allpaths, fs)
+        allpaths, base, fns = _sort_and_analyze_paths(allpaths, fs)
         dataset = pq.ParquetDataset(paths[0], filesystem=fs, filters=filters, **kwargs)
     else:
         # This is a single file.  No danger in gathering statistics
@@ -1776,6 +1784,7 @@ class ArrowLegacyEngine(ArrowDatasetEngine):
                     root_dir=base,
                     engine=cls,
                     out_dir=False,
+                    fs=fs,
                 )
                 if schema is None:
                     schema = metadata.schema.to_arrow_schema()
