@@ -3,12 +3,11 @@ import math
 
 import tlz as toolz
 import warnings
-from ....bytes import core  # noqa
 from fsspec.core import get_fs_token_paths
 from fsspec.implementations.local import LocalFileSystem
 from fsspec.utils import stringify_path
 
-from .utils import _analyze_paths
+from .utils import _sort_and_analyze_paths
 from ...core import DataFrame, new_dd_object
 from ....base import tokenize
 from ....delayed import Delayed
@@ -576,7 +575,7 @@ def to_parquet(
             df.columns = [
                 c if c not in reserved_names else NONE_LABEL for c in df.columns
             ]
-        index_cols = [c for c in set(df.columns).difference(real_cols)]
+        index_cols = [c for c in set(df.columns) - real_cols]
     else:
         # Not writing index - might as well drop it
         df = df.reset_index(drop=True)
@@ -681,6 +680,7 @@ def create_metadata_file(
     split_every=32,
     compute=True,
     compute_kwargs=None,
+    fs=None,
 ):
     """Construct a global _metadata file from a list of parquet files.
 
@@ -722,6 +722,12 @@ def create_metadata_file(
         then a ``dask.delayed`` object is returned for future computation.
     compute_kwargs : dict, optional
         Options to be passed in to the compute method
+    fs : fsspec object, optional
+        File-system instance to use for file handling. If prefixes have
+        been removed from the elements of ``paths`` before calling this
+        function, an ``fs`` argument must be provided to ensure correct
+        behavior on remote file systems ("naked" paths cannot be used
+        to infer file-system information).
     """
 
     # Get engine.
@@ -735,10 +741,14 @@ def create_metadata_file(
         engine = get_engine(engine)
 
     # Process input path list
-    fs, _, paths = get_fs_token_paths(paths, mode="rb", storage_options=storage_options)
-    paths = sorted(paths, key=natural_sort_key)  # numeric rather than glob ordering
+    if fs is None:
+        # Only do this if an fsspec file-system object is not
+        # already defined. The prefixes may already be stripped.
+        fs, _, paths = get_fs_token_paths(
+            paths, mode="rb", storage_options=storage_options
+        )
     ap_kwargs = {"root": root_dir} if root_dir else {}
-    root_dir, fns = _analyze_paths(paths, fs, **ap_kwargs)
+    paths, root_dir, fns = _sort_and_analyze_paths(paths, fs, **ap_kwargs)
     out_dir = root_dir if out_dir is None else out_dir
 
     # Start constructing a raw graph
