@@ -1,13 +1,13 @@
 import numpy as np
 import pandas as pd
-import pandas.util.testing as tm
 
 import pytest
+from concurrent.futures import ThreadPoolExecutor
 from threading import Lock
-from multiprocessing.pool import ThreadPool
 
 import dask.array as da
 import dask.dataframe as dd
+from dask.dataframe._compat import tm
 from dask.dataframe.io.io import _meta_from_array
 from dask.delayed import Delayed, delayed
 
@@ -117,7 +117,6 @@ def test_from_array_with_record_dtype():
 
 def test_from_bcolz_multiple_threads():
     bcolz = pytest.importorskip("bcolz")
-    pool = ThreadPool(processes=5)
 
     def check(i):
         t = bcolz.ctable(
@@ -141,7 +140,8 @@ def test_from_bcolz_multiple_threads():
             dd.from_bcolz(t, chunksize=3).dask
         )
 
-    pool.map(check, range(5))
+    with ThreadPoolExecutor(5) as pool:
+        list(pool.map(check, range(5)))
 
 
 def test_from_bcolz():
@@ -263,18 +263,17 @@ def test_from_pandas_small():
             ddf = dd.from_pandas(df, npartitions=5, sort=sort)
             assert_eq(df, ddf)
 
-            s = pd.Series([0] * i, name="x")
+            s = pd.Series([0] * i, name="x", dtype=int)
             ds = dd.from_pandas(s, npartitions=5, sort=sort)
             assert_eq(s, ds)
 
 
-@pytest.mark.xfail(reason="")
-def test_from_pandas_npartitions_is_accurate():
+@pytest.mark.parametrize("n", [1, 2, 4, 5])
+def test_from_pandas_npartitions_is_accurate(n):
     df = pd.DataFrame(
         {"x": [1, 2, 3, 4, 5, 6], "y": list("abdabd")}, index=[10, 20, 30, 40, 50, 60]
     )
-    for n in [1, 2, 4, 5]:
-        assert dd.from_pandas(df, npartitions=n).npartitions == n
+    assert dd.from_pandas(df, npartitions=n).npartitions <= n
 
 
 def test_from_pandas_series():
@@ -508,7 +507,6 @@ def test_from_dask_array_unknown_chunks():
 
 
 def test_to_bag():
-    pytest.importorskip("dask.bag")
     a = pd.DataFrame(
         {"x": ["a", "b", "c", "d"], "y": [2, 3, 4, 5]},
         index=pd.Index([1.0, 2.0, 3.0, 4.0], name="ind"),
