@@ -9,7 +9,6 @@ from .utils import (
     is_series_like,
     is_index_like,
     is_dataframe_like,
-    PANDAS_GT_0250,
     hash_object_dispatch,
     group_split_dispatch,
 )
@@ -129,6 +128,20 @@ def wrap_var_reduction(array_var, index):
     return array_var
 
 
+def wrap_skew_reduction(array_skew, index):
+    if isinstance(array_skew, np.ndarray) or isinstance(array_skew, list):
+        return pd.Series(array_skew, index=index)
+
+    return array_skew
+
+
+def wrap_kurtosis_reduction(array_kurtosis, index):
+    if isinstance(array_kurtosis, np.ndarray) or isinstance(array_kurtosis, list):
+        return pd.Series(array_kurtosis, index=index)
+
+    return array_kurtosis
+
+
 def var_mixed_concat(numeric_var, timedelta_var, columns):
     vars = pd.concat([numeric_var, timedelta_var])
 
@@ -198,10 +211,9 @@ def describe_nonnumeric_aggregate(stats, name):
         data = [0, 0]
         index = ["count", "unique"]
         dtype = None
-        if PANDAS_GT_0250:
-            data.extend([None, None])
-            index.extend(["top", "freq"])
-            dtype = object
+        data.extend([None, None])
+        index.extend(["top", "freq"])
+        dtype = object
         result = pd.Series(data, index=index, dtype=dtype, name=name)
         return result
 
@@ -247,6 +259,24 @@ def _cum_aggregate_apply(aggregate, x, y):
         return aggregate(x, y)
 
 
+def cumsum_aggregate(x, y):
+    if x is None:
+        return y
+    elif y is None:
+        return x
+    else:
+        return x + y
+
+
+def cumprod_aggregate(x, y):
+    if x is None:
+        return y
+    elif y is None:
+        return x
+    else:
+        return x * y
+
+
 def cummin_aggregate(x, y):
     if is_series_like(x) or is_dataframe_like(x):
         return x.where((x < y) | x.isnull(), y, axis=x.ndim - 1)
@@ -286,8 +316,12 @@ def value_counts_combine(x, sort=True, ascending=False, **groupby_kwargs):
     return x.groupby(level=0, **groupby_kwargs).sum()
 
 
-def value_counts_aggregate(x, sort=True, ascending=False, **groupby_kwargs):
+def value_counts_aggregate(
+    x, sort=True, ascending=False, normalize=False, total_length=None, **groupby_kwargs
+):
     out = value_counts_combine(x, **groupby_kwargs)
+    if normalize:
+        out /= total_length if total_length is not None else out.sum()
     if sort:
         return out.sort_values(ascending=ascending)
     return out
@@ -338,7 +372,7 @@ def pivot_agg(df):
 
 def pivot_sum(df, index, columns, values):
     return pd.pivot_table(
-        df, index=index, columns=columns, values=values, aggfunc="sum"
+        df, index=index, columns=columns, values=values, aggfunc="sum", dropna=False
     )
 
 
@@ -346,7 +380,7 @@ def pivot_count(df, index, columns, values):
     # we cannot determine dtype until concatenationg all partitions.
     # make dtype deterministic, always coerce to np.float64
     return pd.pivot_table(
-        df, index=index, columns=columns, values=values, aggfunc="count"
+        df, index=index, columns=columns, values=values, aggfunc="count", dropna=False
     ).astype(np.float64)
 
 
@@ -541,6 +575,13 @@ def concat_pandas(
 
 
 tolist_dispatch = Dispatch("tolist")
+is_categorical_dtype_dispatch = Dispatch("is_categorical_dtype")
+
+
+def is_categorical_dtype(obj):
+    obj = getattr(obj, "dtype", obj)
+    func = is_categorical_dtype_dispatch.dispatch(type(obj))
+    return func(obj)
 
 
 def tolist(obj):
@@ -553,6 +594,13 @@ def tolist_pandas(obj):
     return obj.tolist()
 
 
+@is_categorical_dtype_dispatch.register(
+    (pd.Series, pd.Index, pd.api.extensions.ExtensionDtype, np.dtype)
+)
+def is_categorical_dtype_pandas(obj):
+    return pd.api.types.is_categorical_dtype(obj)
+
+
 # cuDF may try to import old dispatch functions
 hash_df = hash_object_dispatch
 group_split = group_split_dispatch
