@@ -4,18 +4,20 @@ import warnings
 import pytest
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_datetime64_ns_dtype
 
 import dask.dataframe as dd
-from dask.dataframe._compat import PANDAS_GT_100, PANDAS_VERSION
+from dask.dataframe._compat import PANDAS_GT_100, PANDAS_GT_120, PANDAS_VERSION
 from dask.dataframe.utils import (
     assert_eq,
     assert_dask_graph,
     make_meta,
-    HAS_INT_NA,
-    PANDAS_GT_0250,
 )
 
+try:
+    import scipy
+except ImportError:
+    scipy = None
+
 
 @pytest.mark.slow
 def test_arithmetics():
@@ -615,9 +617,6 @@ def test_frame_series_arithmetic_methods():
 
             assert_eq(l.rdiv(r, fill_value=0), el.rdiv(er, fill_value=0))
             assert_eq(l.rtruediv(r, fill_value=0), el.rtruediv(er, fill_value=0))
-            if not PANDAS_GT_0250:
-                # https://github.com/pandas-dev/pandas/issues/27464
-                assert_eq(l.rfloordiv(r, fill_value=1), el.rfloordiv(er, fill_value=1))
             assert_eq(l.rpow(r, fill_value=0), el.rpow(er, fill_value=0))
             assert_eq(l.rmod(r, fill_value=0), el.rmod(er, fill_value=0))
 
@@ -642,9 +641,6 @@ def test_frame_series_arithmetic_methods():
         assert_eq(l.rmul(r, axis=0), el.rmul(er, axis=0))
         assert_eq(l.rdiv(r, axis=0), el.rdiv(er, axis=0))
         assert_eq(l.rtruediv(r, axis=0), el.rtruediv(er, axis=0))
-        if not PANDAS_GT_0250:
-            # https://github.com/pandas-dev/pandas/issues/27464
-            assert_eq(l.rfloordiv(r, axis=0), el.rfloordiv(er, axis=0))
         assert_eq(l.rmod(r, axis=0), el.rmod(er, axis=0))
         assert_eq(l.rpow(r, axis=0), el.rpow(er, axis=0))
 
@@ -669,9 +665,6 @@ def test_frame_series_arithmetic_methods():
                 assert_eq(l.pow(r, axis=axis), el.pow(er, axis=axis))
                 assert_eq(l.rdiv(r, axis=axis), el.rdiv(er, axis=axis))
                 assert_eq(l.rtruediv(r, axis=axis), el.rtruediv(er, axis=axis))
-                if not PANDAS_GT_0250:
-                    # https://github.com/pandas-dev/pandas/issues/27464
-                    assert_eq(l.rfloordiv(r, axis=axis), el.rfloordiv(er, axis=axis))
                 assert_eq(l.rpow(r, axis=axis), el.rpow(er, axis=axis))
                 assert_eq(l.rmod(r, axis=axis), el.rmod(er, axis=axis))
 
@@ -733,6 +726,19 @@ def test_reductions(split_every):
         assert_eq(dds.max(split_every=split_every), pds.max())
         assert_eq(dds.count(split_every=split_every), pds.count())
 
+        if scipy:
+            # pandas uses unbiased skew, need to correct for that
+            n = pds.shape[0]
+            bias_factor = (n * (n - 1)) ** 0.5 / (n - 2)
+            assert_eq(dds.skew(), pds.skew() / bias_factor)
+
+        if scipy:
+            # pandas uses a bias factor for kurtosis, need to correct for that
+            n = pds.shape[0]
+            factor = ((n - 1) * (n + 1)) / ((n - 2) * (n - 3))
+            offset = (6 * (n - 1)) / ((n - 2) * (n - 3))
+            assert_eq(factor * dds.kurtosis() + offset, pds.kurtosis())
+
         with pytest.warns(None):
             # runtime warnings; https://github.com/dask/dask/issues/2381
             assert_eq(dds.std(split_every=split_every), pds.std())
@@ -988,9 +994,8 @@ def test_reductions_non_numeric_dtypes():
     check_raises(dds, pds, "std")
     check_raises(dds, pds, "var")
     check_raises(dds, pds, "sem")
-    if not PANDAS_GT_0250:
-        # pandas 0.25 added DatetimeIndex.mean. We need to follow.
-        check_raises(dds, pds, "mean")
+    check_raises(dds, pds, "skew")
+    check_raises(dds, pds, "kurtosis")
     assert_eq(dds.nunique(), pds.nunique())
 
     for pds in [
@@ -1005,12 +1010,16 @@ def test_reductions_non_numeric_dtypes():
         assert_eq(dds.min(), pds.min())
         assert_eq(dds.max(), pds.max())
         assert_eq(dds.count(), pds.count())
-        check_raises(dds, pds, "std")
+        if PANDAS_GT_120 and pds.dtype == "datetime64[ns]":
+            # std is implemented for datetimes in pandas 1.2.0, but dask
+            # implementation depends on var which isn't
+            pass
+        else:
+            check_raises(dds, pds, "std")
         check_raises(dds, pds, "var")
         check_raises(dds, pds, "sem")
-        if not (PANDAS_GT_0250 and is_datetime64_ns_dtype(pds.dtype)):
-            # pandas 0.25 added DatetimeIndex.mean. We need to follow
-            check_raises(dds, pds, "mean")
+        check_raises(dds, pds, "skew")
+        check_raises(dds, pds, "kurtosis")
         assert_eq(dds.nunique(), pds.nunique())
 
     pds = pd.Series(pd.timedelta_range("1 days", freq="D", periods=5))
@@ -1019,6 +1028,9 @@ def test_reductions_non_numeric_dtypes():
     assert_eq(dds.min(), pds.min())
     assert_eq(dds.max(), pds.max())
     assert_eq(dds.count(), pds.count())
+    # both pandas and dask skew calculations do not support timedelta
+    check_raises(dds, pds, "skew")
+    check_raises(dds, pds, "kurtosis")
 
     # ToDo: pandas supports timedelta std, dask returns float64
     # assert_eq(dds.std(), pds.std())
@@ -1128,12 +1140,6 @@ def test_reductions_frame_dtypes():
         }
     )
 
-    if HAS_INT_NA:
-        if not PANDAS_GT_0250:
-            # Pandas master is returning NA for IntegerNA.sum() when mixed with other dtypes.
-            # https://github.com/pandas-dev/pandas/issues/27185
-            df["intna"] = pd.array([1, 2, 3, 4, None, 6, 7, 8], dtype=pd.Int64Dtype())
-
     ddf = dd.from_pandas(df, 3)
 
     # TODO: std and mean do not support timedelta dtype
@@ -1155,28 +1161,31 @@ def test_reductions_frame_dtypes():
     assert_eq(df.min(), ddf.min())
     assert_eq(df.max(), ddf.max())
     assert_eq(df.count(), ddf.count())
-    assert_eq(df_no_timedelta.std(), ddf_no_timedelta.std())
-    assert_eq(df_no_timedelta.var(), ddf_no_timedelta.var())
-    if PANDAS_GT_0250:
-        # https://github.com/pandas-dev/pandas/issues/18880
+    if PANDAS_GT_120:
+        # std is implemented for datetimes in pandas 1.2.0, but dask
+        # implementation depends on var which isn't
         assert_eq(
-            df.drop("timedelta", axis=1).var(skipna=False),
-            ddf.drop("timedelta", axis=1).var(skipna=False),
+            df_no_timedelta.drop("dt", axis=1).std(),
+            ddf_no_timedelta.drop("dt", axis=1).std(),
         )
     else:
-        assert_eq(df.var(skipna=False), ddf.var(skipna=False))
+        assert_eq(df_no_timedelta.std(), ddf_no_timedelta.std())
+    assert_eq(df_no_timedelta.var(), ddf_no_timedelta.var())
 
+    df2 = df.drop("timedelta", axis=1)
+    ddf2 = ddf.drop("timedelta", axis=1)
+
+    assert_eq(df2.var(skipna=False), ddf2.var(skipna=False))
     assert_eq(df.sem(), ddf.sem())
-    assert_eq(df_no_timedelta.std(ddof=0), ddf_no_timedelta.std(ddof=0))
-    if PANDAS_GT_0250:
-        # https://github.com/pandas-dev/pandas/issues/18880
-        df2 = df.drop("timedelta", axis=1)
-        ddf2 = ddf.drop("timedelta", axis=1)
-        assert_eq(df2.var(ddof=0), ddf2.var(ddof=0))
-        assert_eq(df2.var(ddof=0, skipna=False), ddf2.var(ddof=0, skipna=False))
+    if PANDAS_GT_120:
+        assert_eq(
+            df_no_timedelta.drop("dt", axis=1).std(ddof=0),
+            ddf_no_timedelta.drop("dt", axis=1).std(ddof=0),
+        )
     else:
-        assert_eq(df.var(ddof=0), ddf.var(ddof=0))
-        assert_eq(df.var(ddof=0, skipna=False), ddf.var(ddof=0, skipna=False))
+        assert_eq(df_no_timedelta.std(ddof=0), ddf_no_timedelta.std(ddof=0))
+    assert_eq(df2.var(ddof=0), ddf2.var(ddof=0))
+    assert_eq(df2.var(ddof=0, skipna=False), ddf2.var(ddof=0, skipna=False))
     assert_eq(df.sem(ddof=0), ddf.sem(ddof=0))
 
     assert_eq(df._get_numeric_data(), ddf._get_numeric_data())
@@ -1313,10 +1322,6 @@ def test_series_comparison_nan(comparison):
     )
 
 
-skip_if_no_intna = pytest.mark.skipif(not HAS_INT_NA, reason="integer na")
-
-
-@skip_if_no_intna
 def test_sum_intna():
     a = pd.Series([1, None, 2], dtype=pd.Int32Dtype())
     b = dd.from_pandas(a, 2)
@@ -1341,8 +1346,8 @@ def test_divmod():
     assert_eq(result[1], expected[1])
 
 
+@pytest.mark.skipif("not scipy")
 def test_moment():
-    scipy = pytest.importorskip("scipy")
     from dask.array import stats
     from dask.array.utils import assert_eq
 
