@@ -1,3 +1,5 @@
+from concurrent.futures import ProcessPoolExecutor
+from functools import partial
 import itertools
 import os
 import random
@@ -432,15 +434,14 @@ def test_set_index_consistent_divisions():
     ddf = ddf.clear_divisions()
 
     ctx = mp.get_context("spawn")
-    pool = ctx.Pool(processes=8)
-    with pool:
-        results = [pool.apply_async(_set_index, (ddf, "x")) for _ in range(100)]
-        divisions_set = set(result.get() for result in results)
+    with ProcessPoolExecutor(8, ctx) as pool:
+        func = partial(_set_index, df=ddf, idx="x")
+        divisions_set = set(pool.map(func, range(100)))
     assert len(divisions_set) == 1
 
 
-def _set_index(df, *args, **kwargs):
-    return df.set_index(*args, **kwargs).divisions
+def _set_index(i, df, idx):
+    return df.set_index(idx).divisions
 
 
 @pytest.mark.parametrize("shuffle", ["disk", "tasks"])
@@ -605,13 +606,26 @@ def test_set_index():
     assert_eq(d5, full.set_index(["b"]))
 
 
-def test_set_index_interpolate():
+@pytest.mark.parametrize("engine", ["pandas", "cudf"])
+def test_set_index_interpolate(engine):
+    if engine == "cudf":
+        # NOTE: engine == "cudf" requires cudf/dask_cudf,
+        # will be skipped by non-GPU CI.
+
+        cudf = pytest.importorskip("cudf")
+        dask_cudf = pytest.importorskip("dask_cudf")
+
     df = pd.DataFrame({"x": [4, 1, 1, 3, 3], "y": [1.0, 1, 1, 1, 2]})
-    d = dd.from_pandas(df, 2)
+
+    if engine == "cudf":
+        gdf = cudf.from_pandas(df)
+        d = dask_cudf.from_cudf(gdf, npartitions=3)
+    else:
+        d = dd.from_pandas(df, 2)
 
     d1 = d.set_index("x", npartitions=3)
     assert d1.npartitions == 3
-    assert set(d1.divisions) == set([1, 2, 3, 4])
+    assert set(d1.divisions) == set([1, 2, 4])
 
     d2 = d.set_index("y", npartitions=3)
     assert d2.divisions[0] == 1.0
@@ -619,14 +633,53 @@ def test_set_index_interpolate():
     assert d2.divisions[3] == 2.0
 
 
-def test_set_index_interpolate_int():
+@pytest.mark.parametrize("engine", ["pandas", "cudf"])
+def test_set_index_interpolate_int(engine):
+    if engine == "cudf":
+        # NOTE: engine == "cudf" requires cudf/dask_cudf,
+        # will be skipped by non-GPU CI.
+
+        cudf = pytest.importorskip("cudf")
+        dask_cudf = pytest.importorskip("dask_cudf")
+
     L = sorted(list(range(0, 200, 10)) * 2)
     df = pd.DataFrame({"x": 2 * L})
-    d = dd.from_pandas(df, 2)
+
+    if engine == "cudf":
+        gdf = cudf.from_pandas(df)
+        d = dask_cudf.from_cudf(gdf, npartitions=2)
+    else:
+        d = dd.from_pandas(df, 2)
+
     d1 = d.set_index("x", npartitions=10)
     assert all(np.issubdtype(type(x), np.integer) for x in d1.divisions)
 
 
+@pytest.mark.parametrize("engine", ["pandas", "cudf"])
+def test_set_index_interpolate_large_uint(engine):
+    if engine == "cudf":
+        # NOTE: engine == "cudf" requires cudf/dask_cudf,
+        # will be skipped by non-GPU CI.
+
+        cudf = pytest.importorskip("cudf")
+        dask_cudf = pytest.importorskip("dask_cudf")
+
+    """This test is for #7304"""
+    df = pd.DataFrame(
+        {"x": np.array([612509347682975743, 616762138058293247], dtype=np.uint64)}
+    )
+
+    if engine == "cudf":
+        gdf = cudf.from_pandas(df)
+        d = dask_cudf.from_cudf(gdf, npartitions=2)
+    else:
+        d = dd.from_pandas(df, 1)
+
+    d1 = d.set_index("x", npartitions=1)
+    assert d1.npartitions == 1
+    assert set(d1.divisions) == set([612509347682975743, 616762138058293247])
+
+
 def test_set_index_timezone():
     s_naive = pd.Series(pd.date_range("20130101", periods=3))
     s_aware = pd.Series(pd.date_range("20130101", periods=3, tz="US/Eastern"))
@@ -1132,3 +1185,13 @@ def test_set_index_nan_partition():
     d[d.a > 1].set_index("a", sorted=True)  # Set sorted index with 0 null partitions
     a = d[d.a > 3].set_index("a", sorted=True)  # Set sorted index with 1 null partition
     assert_eq(a, a)
+
+
+@pytest.mark.parametrize(
+    "npartitions",
+    [10, 1],
+)
+def test_sort_values(npartitions):
+    df = pd.DataFrame({"a": np.random.randint(0, 10, 100)})
+    ddf = dd.from_pandas(df, npartitions=npartitions)
+    assert_eq(ddf.sort_values("a"), df.sort_values("a"))
