@@ -18,7 +18,7 @@ import tlz as toolz
 
 from . import config
 from .utils import ensure_dict, ignoring, stringify
-from .base import is_dask_collection
+from .base import clone_key, flatten, is_dask_collection
 from .core import reverse_dict, keys_in_tasks
 from .utils_test import add, inc  # noqa: F401
 
@@ -33,7 +33,7 @@ def compute_layer_dependencies(layers):
         raise RuntimeError(f"{repr(key)} not found")
 
     all_keys = set(key for layer in layers.values() for key in layer)
-    ret = {k: set() for k in layers.keys()}
+    ret = {k: set() for k in layers}
     for k, v in layers.items():
         for key in keys_in_tasks(all_keys - v.keys(), v.values()):
             ret[k].add(_find_layer_containing_key(key))
@@ -56,7 +56,9 @@ class Layer(collections.abc.Mapping):
     implementations.
     """
 
-    def __init__(self, annotations=None):
+    annotations: Optional[Mapping[str, Any]]
+
+    def __init__(self, annotations: Mapping[str, Any] = None):
         if annotations:
             self.annotations = annotations
         else:
@@ -83,72 +85,6 @@ class Layer(collections.abc.Mapping):
         """
         return self.keys()
 
-    def pack_annotations(self) -> Mapping[str, Any]:
-        """Packs Layer annotations for transmission to scheduler
-
-        Callables annotations are fully expanded over Layer keys, while
-        other values are simply transmitted as is
-
-        Returns
-        -------
-        packed_annotations : dict
-            Packed annotations.
-        """
-        if self.annotations is None:
-            return None
-
-        packed = {}
-
-        for a, v in self.annotations.items():
-            if callable(v):
-                packed[a] = {stringify(k): v(k) for k in self}
-                packed[a]["__expanded_annotations__"] = True
-            else:
-                packed[a] = v
-
-        return packed
-
-    @staticmethod
-    def unpack_annotations(
-        annotations: MutableMapping[str, Any],
-        new_annotations: Mapping[str, Any],
-        keys: Iterable,
-    ) -> None:
-        """
-        Expand a set of layer annotations across a set of keys, then merge those
-        expanded annotations for the layer into an existing annotations mapping.
-        This is not a simple shallow merge because some annotations like retries,
-        priority, workers, etc need to be able to retain keys from different layers.
-        """
-        if new_annotations is None:
-            return None
-
-        expanded = {}
-        keys_stringified = False
-
-        # Expand the new annotations across the keyset
-        for a, v in new_annotations.items():
-            if type(v) is dict and "__expanded_annotations__" in v:
-                # Maybe do a destructive update for efficiency?
-                v = v.copy()
-                del v["__expanded_annotations__"]
-                expanded[a] = v
-            else:
-                if not keys_stringified:
-                    keys = [stringify(k) for k in keys]
-                    keys_stringified = True
-
-                expanded[a] = {k: v for k in keys}
-
-        # Merge the expanded annotations with the existing annotations mapping
-        to_merge = {}
-        for k, v in expanded.items():
-            if isinstance(v, dict) and k in annotations:
-                to_merge[k] = {**annotations[k], **v}
-            else:
-                to_merge[k] = v
-        annotations.update(to_merge)
-
     def cull(
         self, keys: set, all_hlg_keys: Iterable
     ) -> Tuple["Layer", Mapping[Hashable, set]]:
@@ -192,7 +128,7 @@ class Layer(collections.abc.Mapping):
                         seen.add(d)
                         work.add(d)
 
-        return BasicLayer(out), ret_deps
+        return MaterializedLayer(out), ret_deps
 
     def get_dependencies(self, key: Hashable, all_hlg_keys: Iterable) -> set:
         """Get dependencies of `key` in the layer
@@ -211,7 +147,161 @@ class Layer(collections.abc.Mapping):
         """
         return keys_in_tasks(all_hlg_keys, [self[key]])
 
-    def __dask_distributed_pack__(self, client) -> Optional[Any]:
+    def __dask_distributed_annotations_pack__(self) -> Optional[Mapping[str, Any]]:
+        """Packs Layer annotations for transmission to scheduler
+
+        Callables annotations are fully expanded over Layer keys, while
+        other values are simply transmitted as is
+
+        Returns
+        -------
+        packed_annotations : dict
+            Packed annotations.
+        """
+        if self.annotations is None:
+            return None
+
+        packed = {}
+
+        for a, v in self.annotations.items():
+            if callable(v):
+                packed[a] = {stringify(k): v(k) for k in self}
+                packed[a]["__expanded_annotations__"] = True
+            else:
+                packed[a] = v
+
+        return packed
+
+    @staticmethod
+    def __dask_distributed_annotations_unpack__(
+        annotations: MutableMapping[str, Any],
+        new_annotations: Optional[Mapping[str, Any]],
+        keys: Iterable[Hashable],
+    ) -> None:
+        """
+        Unpack a set of layer annotations across a set of keys, then merge those
+        expanded annotations for the layer into an existing annotations mapping.
+
+        This is not a simple shallow merge because some annotations like retries,
+        priority, workers, etc need to be able to retain keys from different layers.
+
+        Parameters
+        ----------
+        annotations: MutableMapping[str, Any], input/output
+            Already unpacked annotations, which are to be updated with the new
+            unpacked annotations
+        new_annotations: Mapping[str, Any], optional
+            New annotations to be unpacked into `annotations`
+        keys: Iterable
+            All keys in the layer.
+        """
+        if new_annotations is None:
+            return
+
+        expanded = {}
+        keys_stringified = False
+
+        # Expand the new annotations across the keyset
+        for a, v in new_annotations.items():
+            if type(v) is dict and "__expanded_annotations__" in v:
+                # Maybe do a destructive update for efficiency?
+                v = v.copy()
+                del v["__expanded_annotations__"]
+                expanded[a] = v
+            else:
+                if not keys_stringified:
+                    keys = [stringify(k) for k in keys]
+                    keys_stringified = True
+
+                expanded[a] = dict.fromkeys(keys, v)
+
+        # Merge the expanded annotations with the existing annotations mapping
+        for k, v in expanded.items():
+            v.update(annotations.get(k, {}))
+        annotations.update(expanded)
+
+    def clone(
+        self,
+        keys: set,
+        seed: Hashable,
+        bind_to: Hashable = None,
+    ) -> "tuple[Layer, bool]":
+        """Clone selected keys in the layer, as well as references to keys in other
+        layers
+
+        Parameters
+        ----------
+        keys
+            Keys to be replaced. This never includes keys not listed by
+            :meth:`get_output_keys`. It must also include any keys that are outside
+            of this layer that may be referenced by it.
+        seed
+            Common hashable used to alter the keys; see :func:`dask.base.clone_key`
+        bind_to
+            Optional key to bind the leaf nodes to. A leaf node here is one that does
+            not reference any replaced keys; in other words it's a node where the
+            replacement graph traversal stops; it may still have dependencies on
+            non-replaced nodes.
+            A bound node will not be computed until after ``bind_to`` has been computed.
+
+        Returns
+        -------
+        - New layer
+        - True if the ``bind_to`` key was injected anywhere; False otherwise
+
+        Notes
+        -----
+        This method should be overridden by subclasses to avoid materializing the layer.
+        """
+        from .graph_manipulation import chunks
+
+        is_leaf: bool
+
+        def clone_value(o):
+            """Variant of distributed.utils_comm.subs_multiple, which allows injecting
+            bind_to
+            """
+            nonlocal is_leaf
+
+            typ = type(o)
+            if typ is tuple and o and callable(o[0]):
+                return (o[0],) + tuple(clone_value(i) for i in o[1:])
+            elif typ is list:
+                return [clone_value(i) for i in o]
+            elif typ is dict:
+                return {k: clone_value(v) for k, v in o.items()}
+            else:
+                try:
+                    if o not in keys:
+                        return o
+                except TypeError:
+                    return o
+                is_leaf = False
+                return clone_key(o, seed)
+
+        dsk_new = {}
+        bound = False
+
+        for key, value in self.items():
+            if key in keys:
+                key = clone_key(key, seed)
+                is_leaf = True
+                value = clone_value(value)
+                if bind_to is not None and is_leaf:
+                    value = (chunks.bind, value, bind_to)
+                    bound = True
+
+            dsk_new[key] = value
+
+        return MaterializedLayer(dsk_new), bound
+
+    def __dask_distributed_pack__(
+        self,
+        all_hlg_keys: Iterable[Hashable],
+        known_key_dependencies: Mapping[Hashable, set],
+        client,
+        client_keys: Iterable[Hashable],
+    ) -> Any:
         """Pack the layer for scheduler communication in Distributed
 
         This method should pack its current state and is called by the Client when
@@ -226,29 +316,89 @@ class Layer(collections.abc.Mapping):
           - All keys must be converted to strings now or when unpacking
           - All tasks must be serialized (see dumps_task())
 
-        Alternatively, the method can return None, which will make Distributed
-        materialize the layer and use a default packing method.
+        The default implementation materialize the layer thus layers such as Blockwise
+        and ShuffleLayer should implement a specialized pack and unpack function in
+        order to avoid materialization.
 
         Parameters
         ----------
+        all_hlg_keys: Iterable[Hashable]
+            All keys in the high level graph
+        known_key_dependencies: Mapping[Hashable, set]
+            Already known dependencies
         client: distributed.Client
             The client calling this function.
+        client_keys : Iterable[Hashable]
+            List of keys requested by the client.
 
         Returns
         -------
         state: Object serializable by msgpack
-            Scheduler compatible state of the layer or None
+            Scheduler compatible state of the layer
         """
-        return None
+        from distributed.client import Future
+        from distributed.utils_comm import unpack_remotedata, subs_multiple
+        from distributed.worker import dumps_task
+        from distributed.utils import CancelledError
+
+        dsk = dict(self)
+
+        # Find aliases not in `client_keys` and substitute all matching keys
+        # with its Future
+        values = {
+            k: v
+            for k, v in dsk.items()
+            if isinstance(v, Future) and k not in client_keys
+        }
+        if values:
+            dsk = subs_multiple(dsk, values)
+
+        # Unpack remote data and record its dependencies
+        dsk = {k: unpack_remotedata(v, byte_keys=True) for k, v in dsk.items()}
+        unpacked_futures = set.union(*[v[1] for v in dsk.values()]) if dsk else set()
+        for future in unpacked_futures:
+            if future.client is not client:
+                raise ValueError(
+                    "Inputs contain futures that were created by another client."
+                )
+            if stringify(future.key) not in client.futures:
+                raise CancelledError(stringify(future.key))
+        unpacked_futures_deps = {}
+        for k, v in dsk.items():
+            if len(v[1]):
+                unpacked_futures_deps[k] = {f.key for f in v[1]}
+        dsk = {k: v[0] for k, v in dsk.items()}
+
+        # Calculate dependencies without re-calculating already known dependencies
+        missing_keys = dsk.keys() - known_key_dependencies.keys()
+        dependencies = {
+            k: keys_in_tasks(all_hlg_keys, [dsk[k]], as_list=False)
+            for k in missing_keys
+        }
+        for k, v in unpacked_futures_deps.items():
+            dependencies[k] = set(dependencies.get(k, ())) | v
+
+        # The scheduler expect all keys to be strings
+        dependencies = {
+            stringify(k): {stringify(dep) for dep in deps}
+            for k, deps in dependencies.items()
+        }
+
+        merged_hlg_keys = all_hlg_keys | dsk.keys()
+        dsk = {
+            stringify(k): stringify(v, exclusive=merged_hlg_keys)
+            for k, v in dsk.items()
+        }
+        dsk = toolz.valmap(dumps_task, dsk)
+        return {"dsk": dsk, "dependencies": dependencies}
 
     @classmethod
     def __dask_distributed_unpack__(
         cls,
         state: Any,
-        dsk: Dict[str, Any],
-        dependencies: MutableMapping[Hashable, set],
-        annotations: Dict[str, Any],
-    ) -> None:
+        dsk: Mapping[str, Any],
+        dependencies: Mapping[str, set],
+    ) -> Dict:
         """Unpack the state of a layer previously packed by __dask_distributed_pack__()
 
         This method is called by the scheduler in Distributed in order to unpack
@@ -263,33 +413,24 @@ class Layer(collections.abc.Mapping):
         ----------
         state: Any
             The state returned by Layer.__dask_distributed_pack__()
-        dsk: dict
+        dsk: Mapping, read-only
             The materialized low level graph of the already unpacked layers
-        dependencies: MutableMapping
+        dependencies: Mapping, read-only
             The dependencies of each key in `dsk`
-        annotations: dict
-            The materialized task annotations
-        """
-        dsk.update(state["dsk"])
-        for k, v in state["dependencies"].items():
-            dependencies[k] = set(dependencies.get(k, ())) | set(v)
 
-        if state["annotations"]:
-            cls.unpack_annotations(
-                annotations, state["annotations"], state["dsk"].keys()
-            )
+        Returns
+        -------
+        unpacked-layer: dict
+            layer_dsk: Mapping[str, Any]
+                Materialized (stringified) graph of the layer
+            layer_deps: Mapping[str, set]
+                Dependencies of each key in `layer_dsk`
+        """
+        return {"dsk": state["dsk"], "deps": state["dependencies"]}
 
     def __reduce__(self):
-        """Default serialization implementation, which materializes the Layer
-
-        This should follow the standard pickle protocol[1] but must always return
-        a tuple and the arguments for the callable object must be compatible with
-        msgpack. This is because Distributed uses msgpack to send Layers to the
-        scheduler.
-
-        [1] <https://docs.python.org/3/library/pickle.html#object.__reduce__>
-        """
-        return (BasicLayer, (dict(self),))
+        """Default serialization implementation, which materializes the Layer"""
+        return (MaterializedLayer, (dict(self),))
 
     def __copy__(self):
         """Default shallow copy implementation"""
@@ -298,10 +439,8 @@ class Layer(collections.abc.Mapping):
         return obj
 
 
-class BasicLayer(Layer):
-    """Basic implementation of `Layer`
-
-    Fully materialized layer implemented by a mapping
+class MaterializedLayer(Layer):
+    """Fully materialized layer of `Layer`
 
     Parameters
     ----------
@@ -405,13 +544,14 @@ class HighLevelGraph(Mapping):
         self,
         layers: Mapping[str, Mapping],
         dependencies: Mapping[str, AbstractSet],
-        key_dependencies: Dict[Hashable, AbstractSet] = None,
+        key_dependencies: Optional[Dict[Hashable, AbstractSet]] = None,
     ):
         self.dependencies = dependencies
         self.key_dependencies = key_dependencies or {}
         # Makes sure that all layers are `Layer`
         self.layers = {
-            k: v if isinstance(v, Layer) else BasicLayer(v) for k, v in layers.items()
+            k: v if isinstance(v, Layer) else MaterializedLayer(v)
+            for k, v in layers.items()
         }
 
     @classmethod
@@ -426,10 +566,7 @@ class HighLevelGraph(Mapping):
                 with ignoring(AttributeError):
                     deps.update({name: set(collection.__dask_layers__())})
             else:
-                try:
-                    [key] = collection.__dask_layers__()
-                except AttributeError:
-                    key = id(graph)
+                key = _get_some_layer_name(collection)
                 layers = {name: layer, key: graph}
                 deps = {name: {key}, key: set()}
         else:
@@ -456,7 +593,7 @@ class HighLevelGraph(Mapping):
         layer : Mapping
             The graph layer itself
         dependencies : List of Dask collections
-            A lit of other dask collections (like arrays or dataframes) that
+            A list of other dask collections (like arrays or dataframes) that
             have graphs themselves
 
         Examples
@@ -485,10 +622,7 @@ class HighLevelGraph(Mapping):
                     with ignoring(AttributeError):
                         deps[name] |= set(collection.__dask_layers__())
                 else:
-                    try:
-                        [key] = collection.__dask_layers__()
-                    except AttributeError:
-                        key = id(graph)
+                    key = _get_some_layer_name(collection)
                     layers[key] = graph
                     deps[name].add(key)
                     deps[key] = set()
@@ -519,8 +653,13 @@ class HighLevelGraph(Mapping):
 
         raise KeyError(key)
 
-    def __len__(self):
-        return len(self.to_dict())
+    def __len__(self) -> int:
+        # NOTE: this will double-count keys that are duplicated between layers, so it's
+        # possible that `len(hlg) > len(hlg.to_dict())`. However, duplicate keys should
+        # not occur through normal use, and their existence would usually be a bug.
+        # So we ignore this case in favor of better performance.
+        # https://github.com/dask/dask/issues/7271
+        return sum(len(layer) for layer in self.layers.values())
 
     def __iter__(self):
         return iter(self.to_dict())
@@ -566,7 +705,10 @@ class HighLevelGraph(Mapping):
         except AttributeError:
             keys: set = set()
             for layer in self.layers.values():
-                keys |= layer.get_output_keys()
+                # Note: don't use `keys |= ...`, because the RHS is a
+                # collections.abc.Set rather than a real set, and this will
+                # cause a whole new set to be constructed.
+                keys.update(layer.get_output_keys())
             self._all_external_keys = keys
             return keys
 
@@ -651,38 +793,53 @@ class HighLevelGraph(Mapping):
         sorted: list
             List of layer names sorted topologically
         """
-        dependencies = copy.deepcopy(self.dependencies)
-        ready = {k for k, v in dependencies.items() if len(v) == 0}
+        degree = {k: len(v) for k, v in self.dependencies.items()}
+        reverse_deps = {k: [] for k in self.dependencies}
+        ready = []
+        for k, v in self.dependencies.items():
+            for dep in v:
+                reverse_deps[dep].append(k)
+            if not v:
+                ready.append(k)
         ret = []
         while len(ready) > 0:
             layer = ready.pop()
             ret.append(layer)
-            del dependencies[layer]
-            for k, v in dependencies.items():
-                v.discard(layer)
-                if len(v) == 0:
-                    ready.add(k)
+            for rdep in reverse_deps[layer]:
+                degree[rdep] -= 1
+                if degree[rdep] == 0:
+                    ready.append(rdep)
         return ret
 
-    def cull(self, keys: set) -> "HighLevelGraph":
+    def cull(self, keys: Iterable) -> "HighLevelGraph":
         """Return new HighLevelGraph with only the tasks required to calculate keys.
 
         In other words, remove unnecessary tasks from dask.
-        ``keys`` may be a single key or list of keys.
+
+        Parameters
+        ----------
+        keys
+            iterable of keys or nested list of keys such as the output of
+            ``__dask_keys__()``
 
         Returns
         -------
         hlg: HighLevelGraph
             Culled high level graph
         """
+        keys_set = set(flatten(keys))
 
         all_ext_keys = self.get_all_external_keys()
         ret_layers = {}
         ret_key_deps = {}
         for layer_name in reversed(self._toposort_layers()):
             layer = self.layers[layer_name]
-            # Let's cull the layer to produce its part of `keys`
-            output_keys = keys & layer.get_output_keys()
+            # Let's cull the layer to produce its part of `keys`.
+            # Note: use .intersection rather than & because the RHS is
+            # a collections.abc.Set rather than a real set, and using &
+            # would take time proportional to the size of the LHS, which
+            # if there is no culling can be much bigger than the RHS.
+            output_keys = keys_set.intersection(layer.get_output_keys())
             if output_keys:
                 culled_layer, culled_deps = layer.cull(output_keys, all_ext_keys)
                 # Update `keys` with all layer's external key dependencies, which
@@ -692,19 +849,46 @@ class HighLevelGraph(Mapping):
                 for d in culled_deps.values():
                     external_deps |= d
                 external_deps -= culled_layer.get_output_keys()
-                keys |= external_deps
+                keys_set |= external_deps
 
                 # Save the culled layer and its key dependencies
                 ret_layers[layer_name] = culled_layer
                 ret_key_deps.update(culled_deps)
 
+        # Converting dict_keys to a real set lets Python optimise the set
+        # intersection to iterate over the smaller of the two sets.
+        ret_layers_keys = set(ret_layers.keys())
         ret_dependencies = {
-            layer_name: self.dependencies[layer_name] & ret_layers.keys()
+            layer_name: self.dependencies[layer_name] & ret_layers_keys
             for layer_name in ret_layers
         }
 
         return HighLevelGraph(ret_layers, ret_dependencies, ret_key_deps)
 
+    def cull_layers(self, layers: Iterable[str]) -> "HighLevelGraph":
+        """Return a new HighLevelGraph with only the given layers and their
+        dependencies. Internally, layers are not modified.
+
+        This is a variant of :meth:`HighLevelGraph.cull` which is much faster and does
+        not risk creating a collision between two layers with the same name and
+        different content when two culled graphs are merged later on.
+
+        Returns
+        -------
+        hlg: HighLevelGraph
+            Culled high level graph
+        """
+        to_visit = set(layers)
+        ret_layers = {}
+        ret_dependencies = {}
+        while to_visit:
+            k = to_visit.pop()
+            ret_layers[k] = self.layers[k]
+            ret_dependencies[k] = self.dependencies[k]
+            to_visit |= ret_dependencies[k] - ret_dependencies.keys()
+
+        return HighLevelGraph(ret_layers, ret_dependencies)
+
     def validate(self):
         # Check dependencies
         for layer_name, deps in self.dependencies.items():
@@ -723,12 +907,12 @@ class HighLevelGraph(Mapping):
         dependencies = compute_layer_dependencies(self.layers)
 
         # Check keys
-        dep_key1 = set(self.dependencies.keys())
-        dep_key2 = set(dependencies.keys())
+        dep_key1 = self.dependencies.keys()
+        dep_key2 = dependencies.keys()
         if dep_key1 != dep_key2:
             raise ValueError(
-                f"incorrect dependencies keys {repr(dep_key1)} "
-                f"expected {repr(dep_key2)}"
+                f"incorrect dependencies keys {set(dep_key1)!r} "
+                f"expected {set(dep_key2)!r}"
             )
 
         # Check values
@@ -739,6 +923,108 @@ class HighLevelGraph(Mapping):
                     f"expected {repr(dependencies[k])}"
                 )
 
+    def __dask_distributed_pack__(self, client, client_keys: Iterable[Hashable]) -> Any:
+        """Pack the high level graph for Scheduler -> Worker communication
+
+        The approach is to delegate the packaging to each layer in the high level graph
+        by calling .__dask_distributed_pack__() and .__dask_distributed_annotations_pack__()
+        on each layer. If the layer doesn't implement packaging, we materialize the
+        layer and pack it.
+
+        Parameters
+        ----------
+        client : distributed.Client
+            The client calling this function.
+        client_keys : Iterable
+            List of keys requested by the client.
+
+        Returns
+        -------
+        data: list of header and payload
+            Packed high level graph serialized by dumps_msgpack
+        """
+        from distributed.protocol.core import dumps_msgpack
+
+        # Dump each layer (in topological order)
+        layers = []
+        for layer in (self.layers[name] for name in self._toposort_layers()):
+            layers.append(
+                {
+                    "__module__": layer.__module__,
+                    "__name__": type(layer).__name__,
+                    "state": layer.__dask_distributed_pack__(
+                        self.get_all_external_keys(),
+                        self.key_dependencies,
+                        client,
+                        client_keys,
+                    ),
+                    "annotations": layer.__dask_distributed_annotations_pack__(),
+                }
+            )
+        return dumps_msgpack({"layers": layers})
+
+    @staticmethod
+    def __dask_distributed_unpack__(packed_hlg, annotations: Mapping[str, Any]) -> Dict:
+        """Unpack the high level graph for Scheduler -> Worker communication
+
+        The approach is to delegate the unpackaging to each layer in the high level graph
+        by calling ..._unpack__() and ..._annotations_unpack__()
+        on each layer.
+
+        Parameters
+        ----------
+        packed_hlg : list of header and payload
+            Packed high level graph serialized by dumps_msgpack
+        annotations : dict
+            A top-level annotations object which may be partially populated,
+            and which may be further filled by annotations from the layers
+            of the packed_hlg.
+
+        Returns
+        -------
+        unpacked-graph: dict
+            dsk: Dict[str, Any]
+                Materialized (stringified) graph of all nodes in the high level graph
+            deps: Dict[str, set]
+                Dependencies of each key in `dsk`
+            annotations: Dict[str, Any]
+                Annotations for `dsk`
+        """
+        from distributed.protocol.core import loads_msgpack
+        from distributed.protocol.serialize import import_allowed_module
+
+        hlg = loads_msgpack(*packed_hlg)
+        dsk = {}
+        deps = {}
+        anno = {}
+
+        # Unpack each layer (in topological order)
+        for layer in hlg["layers"]:
+            # Find the unpack functions
+            if layer["__module__"] is None:  # Default implementation
+                unpack_state = Layer.__dask_distributed_unpack__
+                unpack_anno = Layer.__dask_distributed_annotations_unpack__
+            else:
+                mod = import_allowed_module(layer["__module__"])
+                cls = getattr(mod, layer["__name__"])
+                unpack_state = cls.__dask_distributed_unpack__
+                unpack_anno = cls.__dask_distributed_annotations_unpack__
+
+            # Unpack state into a graph and key dependencies
+            unpacked_layer = unpack_state(layer["state"], dsk, deps)
+            dsk.update(unpacked_layer["dsk"])
+            for k, v in unpacked_layer["deps"].items():
+                deps[k] = deps.get(k, set()) | v
+
+            # Unpack the annotations
+            if annotations and layer["annotations"]:
+                layer_annotations = {**layer["annotations"], **annotations}
+            else:
+                layer_annotations = annotations or layer["annotations"] or None
+            unpack_anno(anno, layer_annotations, unpacked_layer["dsk"].keys())
+
+        return {"dsk": dsk, "deps": deps, "annotations": anno}
+
 
 def to_graphviz(
     hg,
@@ -779,3 +1065,14 @@ def to_graphviz(
             dep_name = name(dep)
             g.edge(dep_name, k_name)
     return g
+
+
+def _get_some_layer_name(collection) -> str:
+    """Somehow get a unique name for a Layer from a non-HighLevelGraph dask mapping"""
+    try:
+        (name,) = collection.__dask_layers__()
+        return name
+    except (AttributeError, ValueError):
+        # collection does not define the optional __dask_layers__ method
+        # or it spuriously returns more than one layer
+        return str(id(collection))
