@@ -1,15 +1,18 @@
 import copyreg
 import multiprocessing
+import os
 import pickle
 import sys
 import traceback
 from functools import partial
 from warnings import warn
 
+import cloudpickle
 from . import config
 from .system import CPU_COUNT
 from .local import reraise, get_async  # TODO: get better get
 from .optimization import fuse, cull
+from .utils import ensure_dict
 
 
 def _reduce_method_descriptor(m):
@@ -19,23 +22,8 @@ def _reduce_method_descriptor(m):
 # type(set.union) is used as a proxy to <class 'method_descriptor'>
 copyreg.pickle(type(set.union), _reduce_method_descriptor)
 
-
-try:
-    import cloudpickle
-
-    _dumps = partial(cloudpickle.dumps, protocol=pickle.HIGHEST_PROTOCOL)
-    _loads = cloudpickle.loads
-except ImportError:
-
-    def _dumps(obj):
-        try:
-            return pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)
-        except (pickle.PicklingError, AttributeError) as exc:
-            raise ModuleNotFoundError(
-                "Please install cloudpickle to use the multiprocessing scheduler"
-            ) from exc
-
-    _loads = pickle.loads
+_dumps = partial(cloudpickle.dumps, protocol=pickle.HIGHEST_PROTOCOL)
+_loads = cloudpickle.loads
 
 
 def _process_get_id():
@@ -56,7 +44,7 @@ def _process_get_id():
 # To enable testing of the ``RemoteException`` class even when tblib is
 # installed, we don't wrap the class in the try block below
 class RemoteException(Exception):
-    """ Remote Exception
+    """Remote Exception
 
     Contains the exception and traceback from a remotely run task
     """
@@ -160,7 +148,7 @@ def get(
     pool=None,
     **kwargs
 ):
-    """ Multiprocessed get function appropriate for Bags
+    """Multiprocessed get function appropriate for Bags
 
     Parameters
     ----------
@@ -171,17 +159,24 @@ def get(
     num_workers : int
         Number of worker processes (defaults to number of cores)
     func_dumps : function
-        Function to use for function serialization
-        (defaults to cloudpickle.dumps if available, otherwise pickle.dumps)
+        Function to use for function serialization (defaults to cloudpickle.dumps)
     func_loads : function
-        Function to use for function deserialization
-        (defaults to cloudpickle.loads if available, otherwise pickle.loads)
+        Function to use for function deserialization (defaults to cloudpickle.loads)
     optimize_graph : bool
         If True [default], `fuse` is applied to the graph before computation.
     """
     pool = pool or config.get("pool", None)
     num_workers = num_workers or config.get("num_workers", None) or CPU_COUNT
     if pool is None:
+        # In order to get consistent hashing in subprocesses, we need to set a
+        # consistent seed for the Python hash algorithm. Unfortunatley, there
+        # is no way to specify environment variables only for the Pool
+        # processes, so we have to rely on environment variables being
+        # inherited.
+        if os.environ.get("PYTHONHASHSEED") in (None, "0"):
+            # This number is arbitrary; it was chosen to commemorate
+            # https://github.com/dask/dask/issues/6640.
+            os.environ["PYTHONHASHSEED"] = "6640"
         context = get_context()
         pool = context.Pool(num_workers, initializer=initialize_worker_process)
         cleanup = True
@@ -189,6 +184,7 @@ def get(
         cleanup = False
 
     # Optimize Dask
+    dsk = ensure_dict(dsk)
     dsk2, dependencies = cull(dsk, keys)
     if optimize_graph:
         dsk3, dependencies = fuse(dsk2, keys, dependencies)
