@@ -6,9 +6,10 @@ import sys
 import time
 from collections import OrderedDict
 
-from tlz import merge
+from tlz import merge, partial, compose, curry
 
 import dask
+import dask.bag as db
 from dask import delayed
 from dask.base import (
     compute,
@@ -25,25 +26,19 @@ from dask.base import (
     named_schedulers,
     get_scheduler,
     collections_to_dsk,
-    get_collection_name,
+    get_collection_names,
+    get_name_from_key,
     replace_name_in_key,
+    clone_key,
 )
 from dask.core import literal
 from dask.delayed import Delayed
-from dask.utils import tmpdir, tmpfile, ignoring
-from dask.utils_test import inc, dec
+from dask.utils import tmpdir, tmpfile
+from dask.utils_test import dec, inc, import_or_none
 from dask.diagnostics import Profiler
 
 
-def import_or_none(path):
-    with ignoring(BaseException):
-        return pytest.importorskip(path)
-    return None
-
-
-tz = pytest.importorskip("tlz")
 da = import_or_none("dask.array")
-db = import_or_none("dask.bag")
 dd = import_or_none("dask.dataframe")
 np = import_or_none("numpy")
 sp = import_or_none("scipy.sparse")
@@ -63,39 +58,24 @@ def f3(a):
 
 
 def test_normalize_function():
-
     assert normalize_function(f2)
 
     assert normalize_function(lambda a: a)
 
-    assert normalize_function(tz.partial(f2, b=2)) == normalize_function(
-        tz.partial(f2, b=2)
-    )
+    assert normalize_function(partial(f2, b=2)) == normalize_function(partial(f2, b=2))
 
-    assert normalize_function(tz.partial(f2, b=2)) != normalize_function(
-        tz.partial(f2, b=3)
-    )
+    assert normalize_function(partial(f2, b=2)) != normalize_function(partial(f2, b=3))
 
-    assert normalize_function(tz.partial(f1, b=2)) != normalize_function(
-        tz.partial(f2, b=2)
-    )
+    assert normalize_function(partial(f1, b=2)) != normalize_function(partial(f2, b=2))
 
-    assert normalize_function(tz.compose(f2, f3)) == normalize_function(
-        tz.compose(f2, f3)
-    )
+    assert normalize_function(compose(f2, f3)) == normalize_function(compose(f2, f3))
 
-    assert normalize_function(tz.compose(f2, f3)) != normalize_function(
-        tz.compose(f2, f1)
-    )
+    assert normalize_function(compose(f2, f3)) != normalize_function(compose(f2, f1))
 
-    assert normalize_function(tz.curry(f2)) == normalize_function(tz.curry(f2))
-    assert normalize_function(tz.curry(f2)) != normalize_function(tz.curry(f1))
-    assert normalize_function(tz.curry(f2, b=1)) == normalize_function(
-        tz.curry(f2, b=1)
-    )
-    assert normalize_function(tz.curry(f2, b=1)) != normalize_function(
-        tz.curry(f2, b=2)
-    )
+    assert normalize_function(curry(f2)) == normalize_function(curry(f2))
+    assert normalize_function(curry(f2)) != normalize_function(curry(f1))
+    assert normalize_function(curry(f2, b=1)) == normalize_function(curry(f2, b=1))
+    assert normalize_function(curry(f2, b=1)) != normalize_function(curry(f2, b=2))
 
 
 def test_tokenize():
@@ -229,7 +209,7 @@ def test_tokenize_numpy_ufunc_consistent():
 
 
 def test_tokenize_partial_func_args_kwargs_consistent():
-    f = tz.partial(f3, f2, c=f1)
+    f = partial(f3, f2, c=f1)
     res = normalize_token(f)
     sol = (
         b"cdask.tests.test_base\nf3\np0\n.",
@@ -548,7 +528,7 @@ def test_unpack_collections():
     repack([(fail, 1)])  # Smoketest results that look like tasks
 
 
-def test_get_collection_name():
+def test_get_collection_names():
     class DummyCollection:
         def __init__(self, dsk, keys):
             self.dask = dsk
@@ -561,37 +541,68 @@ def test_get_collection_name():
             return self.keys
 
     with pytest.raises(TypeError):
-        get_collection_name(object())
+        get_collection_names(object())
     with pytest.raises(TypeError):
-        get_collection_name(DummyCollection(None, []))
-    with pytest.raises(KeyError):
-        get_collection_name(DummyCollection({}, []))
+        get_collection_names(DummyCollection(None, []))
+    # Keys must either be a string or a tuple where the first element is a string
     with pytest.raises(TypeError):
-        # Keys must either be a string or a tuple where the first element is a string
-        get_collection_name(DummyCollection({1: 2}, [1]))
+        get_collection_names(DummyCollection({1: 2}, [1]))
+    with pytest.raises(TypeError):
+        get_collection_names(DummyCollection({(): 1}, [()]))
+    with pytest.raises(TypeError):
+        get_collection_names(DummyCollection({(1,): 1}, [(1,)]))
 
-    assert get_collection_name(DummyCollection({"a-123": 1}, ["a-123"])) == "a-123"
+    assert get_collection_names(DummyCollection({}, [])) == set()
 
     # Arbitrary hashables
     h1 = object()
     h2 = object()
     # __dask_keys__() returns a nested list
     assert (
-        get_collection_name(
-            DummyCollection({("a", h1): 1, ("a", h2): 2}, [[[("a", h1), ("a", h2)]]])
+        get_collection_names(
+            DummyCollection(
+                {("a-1", h1): 1, ("a-1", h2): 2, "b-2": 3, "c": 4},
+                [[[("a-1", h1), ("a-1", h2), "b-2", "c"]]],
+            )
         )
-        == "a"
+        == {"a-1", "b-2", "c"}
     )
 
 
+def test_get_name_from_key():
+    # Arbitrary hashables
+    h1 = object()
+    h2 = object()
+
+    assert get_name_from_key("foo") == "foo"
+    assert get_name_from_key("foo-123"), "foo-123"
+    assert get_name_from_key(("foo-123", h1, h2)) == "foo-123"
+    with pytest.raises(TypeError):
+        get_name_from_key(1)
+    with pytest.raises(TypeError):
+        get_name_from_key(())
+    with pytest.raises(TypeError):
+        get_name_from_key((1,))
+
+
 def test_replace_name_in_keys():
-    assert replace_name_in_key("foo", "bar") == "bar"
-    assert replace_name_in_key("foo-123", "bar-456") == "bar-456"
+    assert replace_name_in_key("foo", {}) == "foo"
+    assert replace_name_in_key("foo", {"bar": "baz"}) == "foo"
+    assert replace_name_in_key("foo", {"foo": "bar", "baz": "asd"}) == "bar"
+    assert replace_name_in_key("foo-123", {"foo-123": "bar-456"}) == "bar-456"
     h1 = object()  # Arbitrary hashables
     h2 = object()
-    assert replace_name_in_key(("foo-123", h1, h2), "bar") == ("bar", h1, h2)
+    assert replace_name_in_key(("foo-123", h1, h2), {"foo-123": "bar"}) == (
+        "bar",
+        h1,
+        h2,
+    )
     with pytest.raises(TypeError):
-        replace_name_in_key(1, "foo")
+        replace_name_in_key(1, {})
+    with pytest.raises(TypeError):
+        replace_name_in_key((), {})
+    with pytest.raises(TypeError):
+        replace_name_in_key((1,), {})
 
 
 class Tuple(DaskMethodsMixin):
@@ -605,9 +616,7 @@ class Tuple(DaskMethodsMixin):
     def __add__(self, other):
         if not isinstance(other, Tuple):
             return NotImplemented  # pragma: nocover
-        name = tokenize(self, other)
-        dsk = {(name, i): k for i, k in enumerate(self._keys + other._keys)}
-        return Tuple(merge(dsk, self._dask, other._dask), list(dsk))
+        return Tuple(merge(self._dask, other._dask), self._keys + other._keys)
 
     def __dask_graph__(self):
         return self._dask
@@ -615,6 +624,9 @@ class Tuple(DaskMethodsMixin):
     def __dask_keys__(self):
         return self._keys
 
+    def __dask_layers__(self):
+        return tuple(get_collection_names(self))
+
     def __dask_tokenize__(self):
         return self._keys
 
@@ -625,9 +637,9 @@ class Tuple(DaskMethodsMixin):
         return Tuple._rebuild, (self._keys,)
 
     @staticmethod
-    def _rebuild(dsk, keys, name=None):
-        if name is not None:
-            keys = [replace_name_in_key(key, name) for key in keys]
+    def _rebuild(dsk, keys, *, rename=None):
+        if rename:
+            keys = [replace_name_in_key(key, rename) for key in keys]
         return Tuple(dsk, keys)
 
 
@@ -636,14 +648,9 @@ def test_custom_collection():
     h1 = object()
     h2 = object()
 
-    # Collections with 2+ keys must have all keys in the format of tuples where the
-    # first element is the same string, referred to as collection name, and the rest are
-    # arbitrary hashables
     dsk = {("x", h1): 1, ("x", h2): 2}
     dsk2 = {("y", h1): (add, ("x", h1), ("x", h2)), ("y", h2): (add, ("y", h1), 1)}
     dsk2.update(dsk)
-
-    # If and only if there is only one top-level key, it can be just a string
     dsk3 = {"z": (add, ("y", h1), ("y", h2))}
     dsk3.update(dsk2)
 
@@ -651,6 +658,8 @@ def test_custom_collection():
     x = Tuple(dsk, [("x", h1), ("x", h2)])
     y = Tuple(dsk2, [("y", h1), ("y", h2)])
     z = Tuple(dsk3, ["z"])
+    # Collection with multiple names
+    t = w + x + y + z
 
     # __slots__ defined on base mixin class propagates
     with pytest.raises(AttributeError):
@@ -661,20 +670,23 @@ def test_custom_collection():
     assert is_dask_collection(x)
     assert is_dask_collection(y)
     assert is_dask_collection(z)
+    assert is_dask_collection(t)
 
     # tokenize
     assert tokenize(w) == tokenize(w)
     assert tokenize(x) == tokenize(x)
     assert tokenize(y) == tokenize(y)
     assert tokenize(z) == tokenize(z)
-    assert len({tokenize(coll) for coll in (w, x, y, z)}) == 4
+    assert tokenize(t) == tokenize(t)
+    # All tokens are unique
+    assert len({tokenize(coll) for coll in (w, x, y, z, t)}) == 5
 
-    # get_collection_name
-    with pytest.raises(KeyError):
-        get_collection_name(w)
-    assert get_collection_name(x) == "x"
-    assert get_collection_name(y) == "y"
-    assert get_collection_name(z) == "z"
+    # get_collection_names
+    assert get_collection_names(w) == set()
+    assert get_collection_names(x) == {"x"}
+    assert get_collection_names(y) == {"y"}
+    assert get_collection_names(z) == {"z"}
+    assert get_collection_names(t) == {"x", "y", "z"}
 
     # compute
     assert w.compute() == ()
@@ -682,7 +694,6 @@ def test_custom_collection():
     assert y.compute() == (3, 4)
     assert z.compute() == (7,)
     assert dask.compute(w, [{"x": x}, y, z]) == ((), [{"x": (1, 2)}, (3, 4), (7,)])
-    t = w + x + y + z
     assert t.compute() == (1, 2, 3, 4, 7)
 
     # persist
@@ -702,19 +713,18 @@ def test_custom_collection():
 
     # __dask_postpersist__ with name change
     rebuild, args = w.__dask_postpersist__()
-    w3 = rebuild({}, *args, name="w3")
+    w3 = rebuild({}, *args, rename={"w": "w3"})
     assert w3.compute() == ()
 
     rebuild, args = x.__dask_postpersist__()
-    x3 = rebuild({("x3", h1): 10, ("x3", h2): 20}, *args, name="x3")
+    x3 = rebuild({("x3", h1): 10, ("x3", h2): 20}, *args, rename={"x": "x3"})
     assert x3.compute() == (10, 20)
 
     rebuild, args = z.__dask_postpersist__()
-    z3 = rebuild({"z3": 70}, *args, name="z3")
+    z3 = rebuild({"z3": 70}, *args, rename={"z": "z3"})
     assert z3.compute() == (70,)
 
 
-@pytest.mark.skipif("not db")
 def test_compute_no_opt():
     # Bag does `fuse` by default. Test that with `optimize_graph=False` that
     # doesn't get called. We check this by using a callback to track the keys
@@ -722,8 +732,8 @@ def test_compute_no_opt():
     from dask.callbacks import Callback
 
     b = db.from_sequence(range(100), npartitions=4)
-    add1 = tz.partial(add, 1)
-    mul2 = tz.partial(mul, 2)
+    add1 = partial(add, 1)
+    mul2 = partial(mul, 2)
     o = b.map(add1).map(mul2)
     # Check that with the kwarg, the optimization doesn't happen
     keys = []
@@ -768,11 +778,11 @@ def test_persist_array():
 
 
 @pytest.mark.skipif("not da")
-def test_persist_array_change_name():
+def test_persist_array_rename():
     a = da.zeros(4, dtype=int, chunks=2)
     rebuild, args = a.__dask_postpersist__()
     dsk = {("b", 0): np.array([1, 2]), ("b", 1): np.array([3, 4])}
-    b = rebuild(dsk, *args, name="b")
+    b = rebuild(dsk, *args, rename={a.name: "b"})
     assert isinstance(b, da.Array)
     assert b.name == "b"
     assert b.__dask_keys__() == [("b", 0), ("b", 1)]
@@ -824,35 +834,35 @@ def test_persist_scalar():
 
 
 @pytest.mark.skipif("not dd")
-def test_persist_dataframe_change_name():
+def test_persist_dataframe_rename():
     df1 = pd.DataFrame({"a": [1, 2, 3, 4], "b": [5, 6, 7, 8]})
     df2 = pd.DataFrame({"a": [2, 3, 5, 6], "b": [6, 7, 9, 10]})
     ddf1 = dd.from_pandas(df1, npartitions=2)
     rebuild, args = ddf1.__dask_postpersist__()
     dsk = {("x", 0): df2.iloc[:2], ("x", 1): df2.iloc[2:]}
-    ddf2 = rebuild(dsk, *args, name="x")
+    ddf2 = rebuild(dsk, *args, rename={ddf1._name: "x"})
     assert ddf2.__dask_keys__() == [("x", 0), ("x", 1)]
     dd.utils.assert_eq(ddf2, df2)
 
 
 @pytest.mark.skipif("not dd")
-def test_persist_series_change_name():
+def test_persist_series_rename():
     ds1 = pd.Series([1, 2, 3, 4])
     ds2 = pd.Series([5, 6, 7, 8])
     dds1 = dd.from_pandas(ds1, npartitions=2)
     rebuild, args = dds1.__dask_postpersist__()
     dsk = {("x", 0): ds2.iloc[:2], ("x", 1): ds2.iloc[2:]}
-    dds2 = rebuild(dsk, *args, name="x")
+    dds2 = rebuild(dsk, *args, rename={dds1._name: "x"})
     assert dds2.__dask_keys__() == [("x", 0), ("x", 1)]
     dd.utils.assert_eq(dds2, ds2)
 
 
 @pytest.mark.skipif("not dd")
-def test_persist_scalar_change_name():
+def test_persist_scalar_rename():
     ds1 = pd.Series([1, 2, 3, 4])
     dds1 = dd.from_pandas(ds1, npartitions=2).min()
     rebuild, args = dds1.__dask_postpersist__()
-    dds2 = rebuild({("x", 0): 5}, *args, name="x")
+    dds2 = rebuild({("x", 0): 5}, *args, rename={dds1._name: "x"})
     assert dds2.__dask_keys__() == [("x", 0)]
     dd.utils.assert_eq(dds2, 5)
 
@@ -881,7 +891,7 @@ def test_compute_dataframe_invalid_unicode():
     dd.from_pandas(df, npartitions=4)
 
 
-@pytest.mark.skipif("not da or not db")
+@pytest.mark.skipif("not da")
 def test_compute_array_bag():
     x = da.arange(5, chunks=2)
     b = db.from_sequence([1, 2, 3])
@@ -974,7 +984,6 @@ def test_visualize_order():
 
 
 def test_use_cloudpickle_to_tokenize_functions_in__main__():
-    pytest.importorskip("cloudpickle")
     from textwrap import dedent
 
     defn = dedent(
@@ -1135,20 +1144,42 @@ def test_persist_delayed_custom_key(key):
 
 
 @pytest.mark.parametrize(
-    "key,new_name,new_key",
-    [("a", "b", "b"), (("a-123", some_hashable), "b-123", ("b-123", some_hashable))],
+    "key,rename,new_key",
+    [
+        ("a", {}, "a"),
+        ("a", {"c": "d"}, "a"),
+        ("a", {"a": "b"}, "b"),
+        (("a-123", some_hashable), {"a-123": "b-123"}, ("b-123", some_hashable)),
+    ],
 )
-def test_persist_delayed_change_name(key, new_name, new_key):
+def test_persist_delayed_rename(key, rename, new_key):
     d = Delayed(key, {key: 1})
     assert d.compute() == 1
     rebuild, args = d.__dask_postpersist__()
-    dp = rebuild({new_key: 2}, *args, name=new_name)
+    dp = rebuild({new_key: 2}, *args, rename=rename)
     assert dp.compute() == 2
     assert dp.key == new_key
     assert dict(dp.dask) == {new_key: 2}
 
 
-@pytest.mark.skipif("not da or not db")
+def test_persist_delayedleaf():
+    x = delayed(1)
+    (xx,) = persist(x)
+    assert isinstance(xx, Delayed)
+    assert xx.compute() == 1
+
+
+def test_persist_delayedattr():
+    class C:
+        x = 1
+
+    x = delayed(C).x
+    (xx,) = persist(x)
+    assert isinstance(xx, Delayed)
+    assert xx.compute() == 1
+
+
+@pytest.mark.skipif("not da")
 def test_persist_array_bag():
     x = da.arange(5, chunks=2) + 1
     b = db.from_sequence([1, 2, 3]).map(inc)
@@ -1170,7 +1201,6 @@ def test_persist_array_bag():
     assert list(b) == list(bb)
 
 
-@pytest.mark.skipif("not db")
 def test_persist_bag():
     a = db.from_sequence([1, 2, 3], npartitions=2).map(lambda x: x * 2)
     assert len(a.__dask_graph__()) == 4
@@ -1180,7 +1210,6 @@ def test_persist_bag():
     db.utils.assert_eq(a, b)
 
 
-@pytest.mark.skipif("not db")
 def test_persist_item():
     a = db.from_sequence([1, 2, 3], npartitions=2).map(lambda x: x * 2).min()
     assert len(a.__dask_graph__()) == 7
@@ -1190,23 +1219,21 @@ def test_persist_item():
     db.utils.assert_eq(a, b)
 
 
-@pytest.mark.skipif("not db")
-def test_persist_bag_change_name():
+def test_persist_bag_rename():
     a = db.from_sequence([1, 2, 3], npartitions=2)
     rebuild, args = a.__dask_postpersist__()
     dsk = {("b", 0): [4], ("b", 1): [5, 6]}
-    b = rebuild(dsk, *args, name="b")
+    b = rebuild(dsk, *args, rename={a.name: "b"})
     assert isinstance(b, db.Bag)
     assert b.name == "b"
     assert b.__dask_keys__() == [("b", 0), ("b", 1)]
     db.utils.assert_eq(b, [4, 5, 6])
 
 
-@pytest.mark.skipif("not db")
 def test_persist_item_change_name():
     a = db.from_sequence([1, 2, 3]).min()
     rebuild, args = a.__dask_postpersist__()
-    b = rebuild({"x": 4}, *args, name="x")
+    b = rebuild({"x": 4}, *args, rename={a.name: "x"})
     assert isinstance(b, db.Item)
     assert b.__dask_keys__() == ["x"]
     db.utils.assert_eq(b, 4)
@@ -1221,7 +1248,6 @@ def test_normalize_function_limited_size():
 
 def test_optimize_globals():
     da = pytest.importorskip("dask.array")
-    db = pytest.importorskip("dask.bag")
 
     x = da.ones(10, chunks=(5,))
 
@@ -1233,7 +1259,7 @@ def test_optimize_globals():
     assert_eq(x + 1, np.ones(10) + 1)
 
     with dask.config.set(array_optimize=optimize_double):
-        assert_eq(x + 1, (np.ones(10) * 2 + 1) * 2)
+        assert_eq(x + 1, (np.ones(10) * 2 + 1) * 2, check_chunks=False)
 
     assert_eq(x + 1, np.ones(10) + 1)
 
@@ -1309,20 +1335,19 @@ def test_callable_scheduler():
     assert called[0]
 
 
+@pytest.mark.flaky(reruns=10, reruns_delay=5)
+@pytest.mark.slow
 @pytest.mark.parametrize("scheduler", ["threads", "processes"])
 def test_num_workers_config(scheduler):
-    pytest.importorskip("cloudpickle")
     # Regression test for issue #4082
 
-    @delayed
-    def f(x):
-        time.sleep(0.5)
-        return x
-
-    a = [f(i) for i in range(5)]
+    f = delayed(pure=False)(time.sleep)
+    # Be generous with the initial sleep times, as process have been observed
+    # to take >0.5s to spin up
+    a = [f(1.0), f(1.0), f(1.0), f(0.1)]
     num_workers = 3
     with dask.config.set(num_workers=num_workers), Profiler() as prof:
-        a = compute(*a, scheduler=scheduler)
+        compute(*a, scheduler=scheduler)
 
     workers = {i.worker_id for i in prof.results}
 
@@ -1337,3 +1362,17 @@ def test_optimizations_ctd():
         dsk2 = collections_to_dsk([x])
 
     assert dsk1 == dsk2
+
+
+def test_clone_key():
+    h = object()  # arbitrary hashable
+    assert clone_key("inc-1-2-3", 123) == "inc-27b6e15b795fcaff169e0e0df14af97a"
+    assert clone_key("x", 123) == "dc2b8d1c184c72c19faa81c797f8c6b0"
+    assert clone_key("x", 456) == "b76f061b547b00d18b9c7a18ccc47e2d"
+    assert clone_key(("sum-1-2-3", h, 1), 123) == (
+        "sum-27b6e15b795fcaff169e0e0df14af97a",
+        h,
+        1,
+    )
+    with pytest.raises(TypeError):
+        clone_key(1, 2)
