@@ -5,14 +5,15 @@ import pickle
 from random import random
 import types
 
-from tlz import merge
 import pytest
+import cloudpickle
+from tlz import merge
 
 import dask
+import dask.bag as db
 from dask import compute
 from dask.delayed import delayed, to_task_dask, Delayed
 from dask.utils_test import inc
-from dask.dataframe.utils import assert_eq
 
 try:
     from operator import matmul
@@ -20,7 +21,7 @@ except ImportError:
     matmul = None
 
 
-class Tuple(object):
+class Tuple:
     __dask_scheduler__ = staticmethod(dask.threaded.get)
 
     def __init__(self, dsk, keys):
@@ -172,6 +173,27 @@ def test_np_dtype_of_delayed():
     assert delayed(np.array([1], dtype="f8")).dtype.compute() == np.dtype("f8")
 
 
+def test_delayed_visualise_warn():
+    # Raise a warning when user calls visualise()
+    # instead of visualize()
+    def inc(x):
+        return x + 1
+
+    z = dask.delayed(inc)(1)
+    z.compute()
+
+    with pytest.warns(
+        UserWarning, match="dask.delayed objects have no `visualise` method"
+    ):
+        z.visualise(file_name="desk_graph.svg")
+
+    # with no args
+    with pytest.warns(
+        UserWarning, match="dask.delayed objects have no `visualise` method"
+    ):
+        z.visualise()
+
+
 def test_delayed_errors():
     a = delayed([1, 2, 3])
     # Immutable
@@ -413,10 +435,8 @@ def test_array_delayed():
     assert val[0, 0].compute() == (arr + arr + 1)[0, 0]
 
     task, dsk = to_task_dask(darr)
-    orig = set(darr.dask)
-    final = set(dsk)
-    assert orig.issubset(final)
-    diff = final.difference(orig)
+    assert not darr.dask.keys() - dsk.keys()
+    diff = dsk.keys() - darr.dask.keys()
     assert len(diff) == 1
 
     delayed_arr = delayed(darr)
@@ -424,7 +444,6 @@ def test_array_delayed():
 
 
 def test_array_bag_delayed():
-    db = pytest.importorskip("dask.bag")
     da = pytest.importorskip("dask.array")
     np = pytest.importorskip("numpy")
 
@@ -485,7 +504,7 @@ def test_delayed_name_on_call():
 
 
 def test_callable_obj():
-    class Foo(object):
+    class Foo:
         def __init__(self, a):
             self.a = a
 
@@ -559,7 +578,7 @@ def test_keys_from_array():
 
 # Mostly copied from https://github.com/pytoolz/toolz/pull/220
 def test_delayed_decorator_on_method():
-    class A(object):
+    class A:
         BASE = 10
 
         def __init__(self, base):
@@ -609,17 +628,16 @@ def test_attribute_of_attribute():
 
 
 def test_check_meta_flag():
+    dd = pytest.importorskip("dask.dataframe")
     from pandas import Series
-    from dask.delayed import delayed
-    from dask.dataframe import from_delayed
 
     a = Series(["a", "b", "a"], dtype="category")
     b = Series(["a", "c", "a"], dtype="category")
     da = delayed(lambda x: x)(a)
     db = delayed(lambda x: x)(b)
 
-    c = from_delayed([da, db], verify_meta=False)
-    assert_eq(c, c)
+    c = dd.from_delayed([da, db], verify_meta=False)
+    dd.utils.assert_eq(c, c)
 
 
 def modlevel_eager(x):
@@ -654,7 +672,29 @@ def test_pickle(f):
     "f", [delayed(modlevel_eager), modlevel_delayed1, modlevel_delayed2]
 )
 def test_cloudpickle(f):
-    cloudpickle = pytest.importorskip("cloudpickle")
     d = f(2)
     d = cloudpickle.loads(cloudpickle.dumps(d, protocol=pickle.HIGHEST_PROTOCOL))
     assert d.compute() == 3
+
+
+def test_dask_layers():
+    d1 = delayed(1)
+    assert d1.dask.layers.keys() == {d1.key}
+    assert d1.dask.dependencies == {d1.key: set()}
+    assert d1.__dask_layers__() == (d1.key,)
+    d2 = modlevel_delayed1(d1)
+    assert d2.dask.layers.keys() == {d1.key, d2.key}
+    assert d2.dask.dependencies == {d1.key: set(), d2.key: {d1.key}}
+    assert d2.__dask_layers__() == (d2.key,)
+
+
+def test_dask_layers_to_delayed():
+    # da.Array.to_delayed squashes the dask graph and causes the layer name not to
+    # match the key
+    da = pytest.importorskip("dask.array")
+    d = da.ones(1).to_delayed()[0]
+    name = d.key[0]
+    assert d.key[1:] == (0,)
+    assert d.dask.layers.keys() == {"delayed-" + name}
+    assert d.dask.dependencies == {"delayed-" + name: set()}
+    assert d.__dask_layers__() == ("delayed-" + name,)
