@@ -1,32 +1,20 @@
+from distutils.version import LooseVersion
 import sys
 import multiprocessing
 from operator import add
 import pickle
-import random
-
-import numpy as np
 
 import pytest
 import dask
 from dask import compute, delayed
-from dask.multiprocessing import get, _dumps, get_context, remote_exception
+from dask.multiprocessing import get, _dumps, _loads, get_context, remote_exception
 from dask.utils_test import inc
 
-
-try:
-    import cloudpickle  # noqa: F401
-
-    has_cloudpickle = True
-except ImportError:
-    has_cloudpickle = False
-
-requires_cloudpickle = pytest.mark.skipif(
-    not has_cloudpickle, reason="requires cloudpickle"
-)
-not_cloudpickle = pytest.mark.skipif(has_cloudpickle, reason="cloudpickle is installed")
+import cloudpickle
 
 
 def unrelated_function_global(a):
+    np = pytest.importorskip("numpy")
     return np.array([a])
 
 
@@ -42,10 +30,9 @@ def test_pickle_globals():
     assert b"numpy" not in b
 
 
-@requires_cloudpickle
 def test_pickle_locals():
-    """Unrelated locals should not be included in serialized bytes
-    """
+    """Unrelated locals should not be included in serialized bytes"""
+    np = pytest.importorskip("numpy")
 
     def unrelated_function_local(a):
         return np.array([a])
@@ -59,6 +46,27 @@ def test_pickle_locals():
     assert b"unrelated_function_local" not in b
 
 
+@pytest.mark.skipif(pickle.HIGHEST_PROTOCOL < 5, reason="requires pickle protocol 5")
+@pytest.mark.skipif(
+    cloudpickle.__version__ < LooseVersion("1.3.0"),
+    reason="requires cloudpickle >= 1.3.0",
+)
+def test_out_of_band_pickling():
+    """Test that out-of-band pickling works"""
+    np = pytest.importorskip("numpy")
+
+    a = np.arange(5)
+
+    l = []
+    b = _dumps(a, buffer_callback=l.append)
+    assert len(l) == 1
+    assert isinstance(l[0], pickle.PickleBuffer)
+    assert memoryview(l[0]) == memoryview(a)
+
+    a2 = _loads(b, buffers=l)
+    assert np.all(a == a2)
+
+
 def bad():
     raise ValueError("12345")
 
@@ -83,40 +91,22 @@ def test_remote_exception():
     assert "traceback-body" in str(a)
 
 
-@requires_cloudpickle
 def test_lambda_with_cloudpickle():
     dsk = {"x": 2, "y": (lambda x: x + 1, "x")}
     assert get(dsk, "y") == 3
 
 
-@not_cloudpickle
-def test_lambda_without_cloudpickle():
-    dsk = {"x": 2, "y": (lambda x: x + 1, "x")}
-    with pytest.raises(ModuleNotFoundError) as e:
-        get(dsk, "y")
-    assert "cloudpickle" in str(e.value)
-
-
 def lambda_result():
     return lambda x: x + 1
 
 
-@requires_cloudpickle
 def test_lambda_results_with_cloudpickle():
     dsk = {"x": (lambda_result,)}
     f = get(dsk, "x")
     assert f(2) == 3
 
 
-@not_cloudpickle
-def test_lambda_results_without_cloudpickle():
-    dsk = {"x": (lambda_result,)}
-    with pytest.raises(ModuleNotFoundError) as e:
-        get(dsk, "x")
-    assert "cloudpickle" in str(e.value)
-
-
-class NotUnpickleable(object):
+class NotUnpickleable:
     def __getstate__(self):
         return ()
 
@@ -165,9 +155,36 @@ def test_optimize_graph_false():
     assert len(keys) == 2
 
 
-@requires_cloudpickle
-@pytest.mark.parametrize("random", [np.random, random])
+def test_works_with_highlevel_graph():
+    """Previously `dask.multiprocessing.get` would accidentally forward
+    `HighLevelGraph` graphs through the dask optimization/scheduling routines,
+    resulting in odd errors. One way to trigger this was to have a
+    non-indexable object in a task. This is just a smoketest to ensure that
+    things work properly even if `HighLevelGraph` objects get passed to
+    `dask.multiprocessing.get`. See https://github.com/dask/dask/issues/7190.
+    """
+
+    class NoIndex:
+        def __init__(self, x):
+            self.x = x
+
+        def __getitem__(self, key):
+            raise Exception("Oh no!")
+
+    x = delayed(lambda x: x)(NoIndex(1))
+    (res,) = get(x.dask, x.__dask_keys__())
+    assert isinstance(res, NoIndex)
+    assert res.x == 1
+
+
+@pytest.mark.parametrize("random", ["numpy", "random"])
 def test_random_seeds(random):
+    if random == "numpy":
+        np = pytest.importorskip("numpy")
+        random = np.random
+    else:
+        import random
+
     @delayed(pure=False)
     def f():
         return tuple(random.randint(0, 10000) for i in range(5))
@@ -192,11 +209,10 @@ def check_for_pytest():
     sys.platform == "win32", reason="Windows doesn't support different contexts"
 )
 def test_custom_context_used_python3_posix():
-    """ The 'multiprocessing.context' config is used to create the pool.
+    """The 'multiprocessing.context' config is used to create the pool.
 
     We assume default is 'spawn', and therefore test for 'fork'.
     """
-    pytest.importorskip("cloudpickle")
     # We check for 'fork' by ensuring subprocess doesn't have modules only
     # parent process should have:
 
@@ -220,7 +236,7 @@ def test_custom_context_used_python3_posix():
     sys.platform == "win32", reason="Windows doesn't support different contexts"
 )
 def test_get_context_using_python3_posix():
-    """ get_context() respects configuration.
+    """get_context() respects configuration.
 
     If default context is changed this test will need to change too.
     """
@@ -233,7 +249,7 @@ def test_get_context_using_python3_posix():
 
 @pytest.mark.skipif(sys.platform != "win32", reason="POSIX supports different contexts")
 def test_custom_context_ignored_elsewhere():
-    """ On Windows, setting 'multiprocessing.context' doesn't explode.
+    """On Windows, setting 'multiprocessing.context' doesn't explode.
 
     Presumption is it's not used since it's unsupported, but mostly we care about
     not breaking anything.
