@@ -175,7 +175,7 @@ def noop_context():
     yield
 
 
-class IndexCallable(object):
+class IndexCallable:
     """Provide getitem syntax for functions
 
     >>> def inc(x):
@@ -407,7 +407,7 @@ def takes_multiple_arguments(func, varargs=True):
     >>> takes_multiple_arguments(f)
     True
 
-    >>> class Thing(object):
+    >>> class Thing:
     ...     def __init__(self, a): pass
     >>> takes_multiple_arguments(Thing)
     False
@@ -445,7 +445,7 @@ def get_named_args(func):
     ]
 
 
-class Dispatch(object):
+class Dispatch:
     """Simple single dispatch."""
 
     def __init__(self, name=None):
@@ -878,7 +878,7 @@ def put_lines(buf, lines):
 _method_cache = {}
 
 
-class methodcaller(object):
+class methodcaller:
     """
     Return a callable object that calls the given method on its operand.
 
@@ -909,7 +909,7 @@ class methodcaller(object):
     __repr__ = __str__
 
 
-class itemgetter(object):
+class itemgetter:
     """
     Return a callable object that gets an item from the operand
 
@@ -932,7 +932,7 @@ class itemgetter(object):
         return type(self) is type(other) and self.index == other.index
 
 
-class MethodCache(object):
+class MethodCache:
     """Attribute access on this object returns a methodcaller for that
     attribute.
 
@@ -950,7 +950,7 @@ class MethodCache(object):
 M = MethodCache()
 
 
-class SerializableLock(object):
+class SerializableLock:
     _locks = WeakValueDictionary()
     """ A Serializable per-process Lock
 
@@ -1053,9 +1053,11 @@ def ensure_dict(d: Mapping[K, V], *, copy: bool = False) -> Dict[K, V]:
     return result
 
 
-class OperatorMethodMixin(object):
+class OperatorMethodMixin:
     """A mixin for dynamically implementing operators"""
 
+    __slots__ = ()
+
     @classmethod
     def _bind_operator(cls, op):
         """ bind operator to this class """
@@ -1109,6 +1111,17 @@ def partial_by_order(*args, **kwargs):
 def is_arraylike(x):
     """Is this object a numpy array or something similar?
 
+    This function tests specifically for an object that already has
+    array attributes (e.g. np.ndarray, dask.array.Array, cupy.ndarray,
+    sparse.COO), **NOT** for something that can be coerced into an
+    array object (e.g. Python lists and tuples). It is meant for dask
+    developers and developers of downstream libraries.
+
+    Note that this function does not correspond with NumPy's
+    definition of array_like, which includes any object that can be
+    coerced into an array (see definition in the NumPy glossary):
+    https://numpy.org/doc/stable/glossary.html
+
     Examples
     --------
     >>> import numpy as np
@@ -1123,11 +1136,18 @@ def is_arraylike(x):
     """
     from .base import is_dask_collection
 
+    is_duck_array = hasattr(x, "__array_function__") or hasattr(x, "__array_ufunc__")
+
     return bool(
         hasattr(x, "shape")
         and isinstance(x.shape, tuple)
         and hasattr(x, "dtype")
         and not any(is_dask_collection(n) for n in x.shape)
+        # We special case scipy.sparse and cupyx.scipy.sparse arrays as having partial
+        # support for them is useful in scenerios where we mostly call `map_partitions`
+        # or `map_blocks` with scikit-learn functions on dask arrays and dask dataframes.
+        # https://github.com/dask/dask/pull/3738
+        and (is_duck_array or "scipy.sparse" in typename(type(x)))
     )
 
 
@@ -1160,6 +1180,11 @@ def is_index_like(s):
     )
 
 
+def is_cupy_type(x):
+    # TODO: avoid explicit reference to CuPy
+    return "cupy" in str(type(x))
+
+
 def natural_sort_key(s):
     """
     Sorting `key` function for performing a natural sort on a collection of
